# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_email_templates -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft <isis@torproject.org> 0xA3ADB67A2CDB8B35
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2013-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________

"""
.. py:module:: bridgedb.email.templates
    :synopsis: Templates for formatting emails sent out by the email
               distributor.

bridgedb.email.templates
========================

Templates for formatting emails sent out by the email distributor.
"""

from __future__ import print_function
from __future__ import unicode_literals

import logging
import os

from datetime import datetime

from bridgedb import strings
from bridgedb.Dist import MAX_EMAIL_RATE
from bridgedb.HTTPServer import TEMPLATE_DIR


def addCommands(template):
    """Add some text telling a client about supported email command, as well as
    which Pluggable Transports are currently available.
    """
    # Tell them about the various email commands:
    cmdlist = []
    cmdlist.append(template.gettext(strings.EMAIL_MISC_TEXT.get(3)))
    for cmd, desc in strings.EMAIL_COMMANDS.items():
        command  = '  '
        command += cmd
        while not len(command) >= 25:  # Align the command descriptions
            command += ' '
        command += template.gettext(desc)
        cmdlist.append(command)

    commands  = "\n".join(cmdlist) + "\n\n"
    # And include the currently supported transports:
    commands += template.gettext(strings.EMAIL_MISC_TEXT.get(5))
    commands += "\n"
    for pt in strings.CURRENT_TRANSPORTS:
        commands += '  ' + pt + "\n"

    return commands

def addGreeting(template, clientName=None, welcome=False):
    greeting = ""

    if not clientName:
        greeting = template.gettext(strings.EMAIL_MISC_TEXT[7])
    else:
        greeting = template.gettext(strings.EMAIL_MISC_TEXT[6]) % clientName

    if greeting:
        if welcome:
            greeting += u' '
            greeting += template.gettext(strings.EMAIL_MISC_TEXT[4])
        greeting += u'\n\n'

    return greeting

def addKeyfile(template):
    filename = os.path.join(TEMPLATE_DIR, 'bridgedb.asc')

    try:
        with open(filename) as fh:
            keyFile  = fh.read()
    except Exception as error:  # pragma: no cover
        logging.exception(error)
        keyFile = u''
    else:
        keyFile += u'\n\n'

    return keyFile

def addBridgeAnswer(template, answer):
    # Give the user their bridges, i.e. the `answer`:
    bridgeLines  = template.gettext(strings.EMAIL_MISC_TEXT[0])
    bridgeLines += u"\n\n"
    bridgeLines += template.gettext(strings.EMAIL_MISC_TEXT[1])
    bridgeLines += u"\n\n"
    bridgeLines += u"%s\n\n" % answer

    return bridgeLines

def addHowto(template):
    """Add help text on how to add bridges to Tor Browser.

    :type template: ``gettext.NullTranslation`` or ``gettext.GNUTranslation``
    :param template: A gettext translations instance, optionally with fallback
        languages set.
    """
    howToTBB  = template.gettext(strings.HOWTO_TBB[1]) % strings.EMAIL_SPRINTF["HOWTO_TBB1"]
    howToTBB += u'\n\n'
    howToTBB += template.gettext(strings.HOWTO_TBB[2])
    howToTBB += u'\n\n'
    howToTBB += u'\n'.join(["> {0}".format(ln) for ln in
                            template.gettext(strings.HOWTO_TBB[3]).split('\n')])
    howToTBB += u'\n\n'
    howToTBB += template.gettext(strings.HOWTO_TBB[4])
    howToTBB += u'\n\n'
    howToTBB += strings.EMAIL_REFERENCE_LINKS.get("HOWTO_TBB1")
    howToTBB += u'\n\n'
    return howToTBB

def addFooter(template, clientAddress=None):
    """Add a footer::

        --
        <3 BridgeDB
       ________________________________________________________________________
       Public Keys: https://bridges.torproject.org/keys

       This email was generated with rainbows, unicorns, and sparkles
       for alice@example.com on Friday, 09 May, 2014 at 18:59:39.


    :type template: ``gettext.NullTranslation`` or ``gettext.GNUTranslation``
    :param template: A gettext translations instance, optionally with fallback
        languages set.
    :type clientAddress: :api:`twisted.mail.smtp.Address`
    :param clientAddress: The client's email address which should be in the
        ``To:`` header of the response email.
    """
    now = datetime.utcnow()
    clientAddr = clientAddress.addrstr

    footer  = u' --\n'
    footer += u' <3 BridgeDB\n'
    footer += u'_' * 70
    footer += u'\n'
    footer += template.gettext(strings.EMAIL_MISC_TEXT[8])
    footer += u': https://bridges.torproject.org/keys\n'
    footer += template.gettext(strings.EMAIL_MISC_TEXT[9]) \
              % (clientAddr,
                 now.strftime('%A, %d %B, %Y'),
                 now.strftime('%H:%M:%S'))
    footer += u'\n'

    return footer

def buildKeyMessage(template, clientAddress=None):
    message  = addKeyfile(template)
    message += addFooter(template, clientAddress)
    return message

def buildWelcomeText(template, clientAddress=None):
    sections = []
    sections.append(addGreeting(template, clientAddress.local, welcome=True))

    commands = addCommands(template)
    sections.append(commands)

    # Include the same messages as the homepage of the HTTPS distributor:
    welcome  = template.gettext(strings.WELCOME[0]) % strings.EMAIL_SPRINTF["WELCOME0"]
    welcome += template.gettext(strings.WELCOME[1])
    welcome += template.gettext(strings.WELCOME[2]) % strings.EMAIL_SPRINTF["WELCOME2"]
    sections.append(welcome)

    message  = u"\n\n".join(sections)
    # Add the markdown links at the end:
    message += strings.EMAIL_REFERENCE_LINKS.get("WELCOME0")
    message += u"\n\n"
    message += addFooter(template, clientAddress)

    return message

def buildAnswerMessage(template, clientAddress=None, answer=None):
    try:
        message  = addGreeting(template, clientAddress.local)
        message += addBridgeAnswer(template, answer)
        message += addHowto(template)
        message += u'\n\n'
        message += addCommands(template)
        message += u'\n\n'
        message += addFooter(template, clientAddress)
    except Exception as error:  # pragma: no cover
        logging.error("Error while formatting email message template:")
        logging.exception(error)

    return message

def buildSpamWarning(template, clientAddress=None):
    message = addGreeting(template, clientAddress.local)

    try:
        message += template.gettext(strings.EMAIL_MISC_TEXT[0])
        message += u"\n\n"
        message += template.gettext(strings.EMAIL_MISC_TEXT[2]) \
                   % str(MAX_EMAIL_RATE / 3600)
        message += u"\n\n"
        message += addFooter(template, clientAddress)
    except Exception as error:  # pragma: no cover
        logging.error("Error while formatting email spam template:")
        logging.exception(error)

    return message



# -*- coding: utf-8 -*-
"""sitecustomize ― Handles potential loading of extra code when Python starts.

**Module Usage:**

This is normally (this seems not to work with Twisted, for as-of-yet unknown
reasons) useful for using :mod:`coverage` to measure code execution in spawned
subprocesses in the following way:

 1. Set the environment variable ``COVERAGE_PROCESS_START`` to the absolute
    path of the coverage config file. If you are in the top-level of the
    bridgedb repo, do:

        $ export COVERAGE_PROCESS_START="${PWD}/.coveragerc"

 2. In that coverage config file, in the ``[run]`` section, set 
    ``parallel = True``.

 3. Run coverage. From the top-level of the bridgedb repo, try doing:

        $ make reinstall && \
            coverage run $(which trial) ./lib/bridgedb/test/test_* && \
            coverage combine && coverage report && coverage html

If ``COVERAGE_PROCESS_START`` is not set, this code does nothing,
``[run] parallel`` should be set to ``False``, and coverage can be run by
leaving out the ``coverage combine`` portion of the above command.

To view the output HTML coverage data, open
``path/to/bridgedb_repo/doc/coverage_html/index.html`` in a browser.
"""

import coverage
coverage.process_startup()



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_parse_versions ; -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2014-2015 Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, all entities within the AUTHORS file
# :license: see included LICENSE for information

"""Parsers for Tor version number strings.

.. py:module:: bridgedb.parse.versions
    :synopsis: Parsers for Tor version number strings.

bridgedb.parse.versions
=======================
::

  Version - Holds, parses, and does comparison operations for package
            version numbers.
..
"""

from twisted import version as _txversion

# The twisted.python.util.Version class was moved in Twisted==14.0.0 to
# twisted.python.versions.Version:
if _txversion.major >= 14:
    from twisted.python.versions import Version as _Version
else:
    from twisted.python.util import Version as _Version


class InvalidVersionStringFormat(ValueError):
    """Raised when a version string is not in a parseable format."""


class Version(_Version):
    """Holds, parses, and does comparison operations for version numbers.

    :attr str package: The package name, if available.
    :attr int major: The major version number.
    :attr int minor: The minor version number.
    :attr int micro: The micro version number.
    :attr str prerelease: The **prerelease** specifier isn't always present,
        though when it is, it's usually separated from the main
        ``major.minor.micro`` part of the version string with a ``-``, ``+``,
        or ``#`` character. Sometimes the **prerelease** is another number,
        although often it can be a word specifying the release state,
        i.e. ``+alpha``, ``-rc2``, etc.
    """

    def __init__(self, version, package=None):
        """Create a version object.

        Comparisons may be computed between instances of :class:`Version`s.

        >>> from bridgedb.parse.versions import Version
        >>> v1 = Version("0.2.3.25", package="tor")
        >>> v1.base()
        '0.2.3.25'
        >>> v1.package
        'tor'
        >>> v2 = Version("0.2.5.1-alpha", package="tor")
        >>> v2
        Version(package=tor, major=0, minor=2, micro=5, prerelease=1-alpha)
        >>> v1 == v2
        False
        >>> v2 > v1
        True

        :param str version: A Tor version string specifier, i.e. one taken
            from either the ``client-versions`` or ``server-versions`` lines
            within a Tor ``cached-consensus`` file.
        :param str package: The package or program which we are creating a
            version number for.
        """
        if version.find('.') == -1:
            raise InvalidVersionStringFormat(
                "Invalid delimiters in version string: %r" % version)

        package = package if package is not None else str()
        major, minor, micro = [int() for _ in range(3)]
        prerelease = str()
        components = version.split('.')
        if len(components) > 0:
            try:
                prerelease = str(components.pop())
                micro      = int(components.pop())
                minor      = int(components.pop())
                major      = int(components.pop())
            except IndexError:
                pass
        super(Version, self).__init__(package, major, minor, micro, prerelease)

    def base(self):
        """Get the base version number (with prerelease).

        :rtype: string
        :returns: A version number, without the package/program name, and with
            the prefix (if available). For example: '0.2.5.1-alpha'.
        """
        pre = self.getPrefixedPrerelease()
        return '%s.%s.%s%s' % (self.major, self.minor, self.micro, pre)

    def getPrefixedPrerelease(self, separator='.'):
        """Get the prerelease string, prefixed by the separator ``prefix``.

        :param string separator: The separator to use between the rest of the
            version string and the :attr:`prerelease` string.
        :rtype: string
        :returns: The separator plus the ``prefix``, i.e. '.1-alpha'.
        """
        pre = ''
        if self.prerelease is not None:
            pre = separator + self.prerelease
        return pre

    def __repr__(self):
        prerelease = self.getPrefixedPrerelease('')
        return '%s(package=%s, major=%s, minor=%s, micro=%s, prerelease=%s)' \
            % (str(self.__class__.__name__),
               str(self.package),
               str(self.major),
               str(self.minor),
               str(self.micro),
               str(prerelease))



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_configure -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: please see the AUTHORS file for attributions
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2013-2015, Matthew Finkel
#             (c) 2007-2015, Nick Mathewson
#             (c) 2007-2015, The Tor Project, Inc.
# :license: see LICENSE for licensing information

"""Utilities for dealing with configuration files for BridgeDB."""

import logging
import os


def loadConfig(configFile=None, configCls=None):
    """Load configuration settings on top of the current settings.

    All pathnames and filenames within settings in the ``configFile`` will be
    expanded, and their expanded values will be stored in the returned
    :class:`config <Conf>` object.

    ** Note: **
    On the strange-looking use of
      ``exec compile(open(configFile).read(), '<string>', 'exec') in dict()``
    in this function:

    The contents of the config file should be compiled first, and then
    ``exec``ed -- not ``execfile``! -- in order to get the contents of the
    config file to exist within the scope of the configuration dictionary.
    Otherwise, Python *will* default_ to executing the config file directly
    within the ``globals()`` scope.

    Additionally, it's roughly 20-30 times faster_ to use the ``compile``
    builtin on a string (the contents of the file) before ``exec``ing it, than
    using ``execfile`` directly on the file.

    .. _default: http://stackoverflow.com/q/17470193
    .. _faster: http://lucumr.pocoo.org/2011/2/1/exec-in-python/

    :ivar boolean itsSafeToUseLogging: This is called in
        :func:`~bridgedb.Main.run` before
        :func:`bridgedb.safelog.configureLogging`. When called from
        :func:`~bridgedb.Main.run`, the **configCls** parameter is not given,
        because that is the first time that a :class:`Conf` is created. If a
        :class:`logging.Logger` is created in this function, then logging will
        not be correctly configured, therefore, if the **configCls** parameter
        is not given, then it's the first time this function has been called
        and it is therefore not safe to make calls to the logging module.
    :type: configFile: string or None
    :param configFile: If given, the filename of the config file to load.
    :type configCls: :class:`Conf` or None
    :param configCls: The current configuration instance, if one already
        exists.
    :returns: A new :class:`configuration <bridgedb.configure.Conf>`, with the
        old settings as defaults, and the settings from the **configFile** (if
        given) overriding those defaults.
    """
    itsSafeToUseLogging = False
    configuration = {}

    if configCls:
        itsSafeToUseLogging = True
        oldConfig = configCls.__dict__
        configuration.update(**oldConfig)  # Load current settings
        logging.info("Reloading over in-memory configurations...")

    conffile = configFile
    if (configFile is None) and ('CONFIG_FILE' in configuration):
        conffile = configuration['CONFIG_FILE']

    if conffile is not None:
        if itsSafeToUseLogging:
            logging.info("Loading settings from config file: '%s'" % conffile)
        compiled = compile(open(conffile).read(), '<string>', 'exec')
        exec compiled in configuration

    if itsSafeToUseLogging:
        logging.debug("New configuration settings:")
        logging.debug("\n".join(["{0} = {1}".format(key, value)
                                 for key, value in configuration.items()]))

    # Create a :class:`Conf` from the settings stored within the local scope
    # of the ``configuration`` dictionary:
    config = Conf(**configuration)

    # We want to set the updated/expanded paths for files on the ``config``,
    # because the copy of this config, `state.config` is used later to compare
    # with a new :class:`Conf` instance, to see if there were any changes.
    #
    # See :meth:`bridgedb.persistent.State.useUpdatedSettings`.

    for attr in ["PROXY_LIST_FILES", "BRIDGE_FILES", "EXTRA_INFO_FILES"]:
        setting = getattr(config, attr, None)
        if setting is None:  # pragma: no cover
            setattr(config, attr, []) # If they weren't set, make them lists
        else:
            setattr(config, attr, # If they were set, expand the paths:
                    [os.path.abspath(os.path.expanduser(f)) for f in setting])

    for attr in ["DB_FILE", "DB_LOG_FILE", "MASTER_KEY_FILE", "PIDFILE",
                 "ASSIGNMENTS_FILE", "HTTPS_CERT_FILE", "HTTPS_KEY_FILE",
                 "LOG_FILE", "STATUS_FILE", "COUNTRY_BLOCK_FILE",
                 "GIMP_CAPTCHA_DIR", "GIMP_CAPTCHA_HMAC_KEYFILE",
                 "GIMP_CAPTCHA_RSA_KEYFILE", "EMAIL_GPG_HOMEDIR",
                 "EMAIL_GPG_PASSPHRASE_FILE"]:
        setting = getattr(config, attr, None)
        if setting is None:
            setattr(config, attr, setting)
        else:
            setattr(config, attr, os.path.abspath(os.path.expanduser(setting)))

    for attr in ["FORCE_PORTS", "FORCE_FLAGS", "NO_DISTRIBUTION_COUNTRIES"]:
        setting = getattr(config, attr, []) # Default to empty lists
        setattr(config, attr, setting)

    for domain in config.EMAIL_DOMAINS:
        config.EMAIL_DOMAIN_MAP[domain] = domain

    if conffile: # Store the pathname of the config file, if one was used
        config.CONFIG_FILE = os.path.abspath(os.path.expanduser(conffile))

    return config


class Conf(object):
    """A configuration object.  Holds unvalidated attributes."""
    def __init__(self, **attrs):
        for key, value in attrs.items():
            if key == key.upper():
                if not key.startswith('__'):
                    self.__dict__[key] = value



# BridgeDB by Nick Mathewson.
# Copyright (c) 2007-2009, The Tor Project, Inc.
# See LICENSE for licensing information

"""These are legacy integration and unittests which historically lived at
``lib/bridgedb/Tests.py``. They have been moved here to keep the test code
separate from the production codebase.
"""

from __future__ import print_function

import doctest
import os
import random
import sqlite3
import tempfile
import unittest
import warnings
import time
from datetime import datetime

import bridgedb.Bridges
import bridgedb.Main
import bridgedb.Dist
import bridgedb.schedule
import bridgedb.Storage
import re
import ipaddr

from bridgedb.Filters import filterBridgesByIP4
from bridgedb.Filters import filterBridgesByIP6
from bridgedb.Filters import filterBridgesByTransport
from bridgedb.Filters import filterBridgesByNotBlockedIn

from bridgedb.Stability import BridgeHistory

from bridgedb.parse import addr
from bridgedb.test import deprecated_networkstatus as networkstatus

from math import log

def suppressWarnings():
    warnings.filterwarnings('ignore', '.*tmpnam.*')

def randomIP():
    if random.choice(xrange(2)):
        return randomIP4()
    return randomIP6()

def randomIP4():
    return ipaddr.IPv4Address(random.getrandbits(32))

def randomIP4String():
    return randomIP4().compressed

def randomIP6():
    return ipaddr.IPv6Address(random.getrandbits(128))

def randomIP6String():
    return bracketIP6(randomIP6().compressed)

def randomIPString():
    if random.choice(xrange(2)):
        return randomIP4String()
    return randomIP6String()

def bracketIP6(ip):
    """Put brackets around an IPv6 address, just as tor does."""
    return "[%s]" % ip

def random16IP():
    upper = "123.123." # same 16
    lower = ".".join([str(random.randrange(1,256)) for _ in xrange(2)]) 
    return upper+lower

def randomPort():
    return random.randint(1,65535)

def randomPortSpec():
    """
    returns a random list of ports
    """
    ports = []
    for i in range(0,24):
        ports.append(random.randint(1,65535))
    ports.sort(reverse=True)

    portspec = ""
    for i in range(0,16):
        portspec += "%d," % random.choice(ports)
    portspec = portspec.rstrip(',') #remove trailing ,
    return portspec

def randomCountry():
    countries = ['us', 'nl', 'de', 'cz', 'sk', 'as', 'si', 'it']
    #XXX: load from geoip
    return random.choice(countries)

def randomCountrySpec():
    countries = ['us', 'nl', 'de', 'cz', 'sk', 'as', 'si', 'it']
    #XXX: load from geoip
    spec = ""
    choices = []
    for i in xrange(10):
        choices.append(random.choice(countries))
    choices = set(choices) #dedupe
    choices = list(choices)
    spec += ",".join(choices)
    return spec

def fakeBridge(orport=8080, running=True, stable=True, or_addresses=False,
        transports=False):
    nn = "bridge-%s"%random.randrange(0,1000000)
    ip = ipaddr.IPAddress(randomIP4())
    fp = "".join([random.choice("0123456789ABCDEF") for _ in xrange(40)])
    b = bridgedb.Bridges.Bridge(nn,ip,orport,fingerprint=fp)
    b.setStatus(running, stable)

    oraddrs = []
    if or_addresses:
        for i in xrange(8):
            # Only add or_addresses if they are valid. Otherwise, the test
            # will randomly fail if an invalid address is chosen:
            address = randomIP4String()
            portlist = addr.PortList(randomPortSpec())
            if addr.isValidIP(address):
                oraddrs.append((address, portlist,))

    for address, portlist in oraddrs:
        networkstatus.parseALine("{0}:{1}".format(address, portlist))
        try:
            portlist.add(b.or_addresses[address])
        except KeyError:
            pass
        finally:
            b.or_addresses[address] = portlist

    if transports:
        for i in xrange(0,8):
            b.transports.append(bridgedb.Bridges.PluggableTransport(b,
                random.choice(["obfs", "obfs2", "pt1"]),
                randomIP(), randomPort()))
    return b

def fakeBridge6(orport=8080, running=True, stable=True, or_addresses=False,
        transports=False):
    nn = "bridge-%s"%random.randrange(0,1000000)
    ip = ipaddr.IPAddress(randomIP6())
    fp = "".join([random.choice("0123456789ABCDEF") for _ in xrange(40)])
    b = bridgedb.Bridges.Bridge(nn,ip,orport,fingerprint=fp)
    b.setStatus(running, stable)

    oraddrs = []
    if or_addresses:
        for i in xrange(8):
            # Only add or_addresses if they are valid. Otherwise, the test
            # will randomly fail if an invalid address is chosen:
            address = randomIP6()
            portlist = addr.PortList(randomPortSpec())
            if addr.isValidIP(address):
                address = bracketIP6(address)
                oraddrs.append((address, portlist,))

    for address, portlist in oraddrs:
        networkstatus.parseALine("{0}:{1}".format(address, portlist))
        try:
            portlist.add(b.or_addresses[address])
        except KeyError:
            pass
        finally:
            b.or_addresses[address] = portlist

            try:
                portlist.add(b.or_addresses[address])
            except KeyError:
                pass
            finally:
                b.or_addresses[address] = portlist

    if transports:
        for i in xrange(0,8):
            b.transports.append(bridgedb.Bridges.PluggableTransport(b,
                random.choice(["obfs", "obfs2", "pt1"]),
                randomIP(), randomPort()))

    return b

def fake16Bridge(orport=8080, running=True, stable=True):
    nn = "bridge-%s"%random.randrange(0,1000000)
    ip = random16IP()
    fp = "".join([random.choice("0123456789ABCDEF") for _ in xrange(40)])
    b = bridgedb.Bridges.Bridge(nn,ip,orport,fingerprint=fp)
    b.setStatus(running, stable)
    return b

simpleDesc = "router Unnamed %s %s 0 9030\n"\
"opt fingerprint DEAD BEEF F00F DEAD BEEF F00F DEAD BEEF F00F DEAD\n"\
"opt @purpose bridge\n"
orAddress = "or-address %s:%s\n"
def gettimestamp():
    ts = time.strftime("%Y-%m-%d %H:%M:%S")
    return "opt published %s\n" % ts

class RhymesWith255Category:
    def contains(self, ip):
        return ip.endswith(".255")

class EmailBridgeDistTests(unittest.TestCase):
    def setUp(self):
        self.fd, self.fname = tempfile.mkstemp()
        self.db = bridgedb.Storage.Database(self.fname)
        bridgedb.Storage.setDB(self.db)
        self.cur = self.db._conn.cursor()

    def tearDown(self):
        self.db.close()
        os.close(self.fd)
        os.unlink(self.fname)

    def testEmailRateLimit(self):
        db = self.db
        EMAIL_DOMAIN_MAP = {'example.com':'example.com'}
        d = bridgedb.Dist.EmailBasedDistributor(
                "Foo",
                {'example.com': 'example.com',
                    'dkim.example.com': 'dkim.example.com'},
                {'example.com': [], 'dkim.example.com': ['dkim']})
        for _ in xrange(256):
            d.insert(fakeBridge())
        d.getBridgesForEmail('abc@example.com', 1, 3)
        self.assertRaises(bridgedb.Dist.TooSoonEmail,
                d.getBridgesForEmail, 'abc@example.com', 1, 3)
        self.assertRaises(bridgedb.Dist.IgnoreEmail,
                d.getBridgesForEmail, 'abc@example.com', 1, 3)

    def testUnsupportedDomain(self):
        db = self.db
        self.assertRaises(bridgedb.parse.addr.UnsupportedDomain,
                          bridgedb.parse.addr.normalizeEmail,
                          'bad@email.com',
                          {'example.com':'example.com'},
                          {'example.com':[]})

class IPBridgeDistTests(unittest.TestCase):
    def dumbAreaMapper(self, ip):
        return ip
    def testBasicDist(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
        for _ in xrange(256):
            d.insert(fakeBridge())
        n = d.getBridgesForIP("1.2.3.4", "x", 2)
        n2 = d.getBridgesForIP("1.2.3.4", "x", 2)
        self.assertEquals(n, n2)

    def testDistWithCategories(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo",
                                             [RhymesWith255Category()])
        assert len(d.categories) == 1
        for _ in xrange(256):
            d.insert(fakeBridge())

        for _ in xrange(256):
            # Make sure that the categories do not overlap
            f = lambda: ".".join([str(random.randrange(1,255)) for _ in xrange(4)])
            g = lambda: ".".join([str(random.randrange(1,255)) for _ in xrange(3)] + ['255'])
            n = d.getBridgesForIP(g(), "x", 10)
            n2 = d.getBridgesForIP(f(), "x", 10)

            assert(len(n) > 0)
            assert(len(n2) > 0)

            for b in n:
                assert (b not in n2)

            for b in n2:
                assert (b not in n)

    #XXX: #6175 breaks this test!
    #def testDistWithPortRestrictions(self):
    #    param = bridgedb.Bridges.BridgeRingParameters(needPorts=[(443, 1)])
    #    d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Baz",
    #                                         answerParameters=param)
    #    for _ in xrange(32):
    #        d.insert(fakeBridge(443))
    #    for _ in range(256):
    #        d.insert(fakeBridge())
    #    for _ in xrange(32):
    #        i = randomIP()
    #        n = d.getBridgesForIP(i, "x", 5)
    #        count = 0
    #        fps = {}
    #        for b in n:
    #            fps[b.getID()] = 1
    #            if b.orport == 443:
    #                count += 1
    #        self.assertEquals(len(fps), len(n))
    #        self.assertEquals(len(fps), 5)
    #        self.assertTrue(count >= 1)

    #def testDistWithFilter16(self):
    #    d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
    #    for _ in xrange(256):
    #        d.insert(fake16Bridge())
    #    n = d.getBridgesForIP("1.2.3.4", "x", 10)

    #    slash16s = dict()
    #    for bridge in n:
    #        m = re.match(r'(\d+\.\d+)\.\d+\.\d+', bridge.ip)
    #        upper16 = m.group(1)
    #        self.assertTrue(upper16 not in slash16s)
    #        slash16s[upper16] = True

    def testDistWithFilterIP6(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
        for _ in xrange(250):
            d.insert(fakeBridge6(or_addresses=True))
            d.insert(fakeBridge(or_addresses=True))

        for i in xrange(500):
            bridges = d.getBridgesForIP(randomIP4String(),
                                        "faketimestamp",
                                        bridgeFilterRules=[filterBridgesByIP6])
            bridge = random.choice(bridges)
            bridge_line = bridge.getConfigLine(addressClass=ipaddr.IPv6Address)
            address, portlist = networkstatus.parseALine(bridge_line)
            assert type(ipaddr.IPAddress(address)) is ipaddr.IPv6Address
            assert filterBridgesByIP6(random.choice(bridges))

    def testDistWithFilterIP4(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
        for _ in xrange(250):
            d.insert(fakeBridge6(or_addresses=True))
            d.insert(fakeBridge(or_addresses=True))

        for i in xrange(500):
            bridges = d.getBridgesForIP(randomIP4String(),
                                        "faketimestamp",
                                        bridgeFilterRules=[filterBridgesByIP4])
            bridge = random.choice(bridges)
            bridge_line = bridge.getConfigLine(addressClass=ipaddr.IPv4Address)
            address, portlist = networkstatus.parseALine(bridge_line)
            assert type(ipaddr.IPAddress(address)) is ipaddr.IPv4Address
            assert filterBridgesByIP4(random.choice(bridges))

    def testDistWithFilterBoth(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
        for _ in xrange(250):
            d.insert(fakeBridge6(or_addresses=True))
            d.insert(fakeBridge(or_addresses=True))

        for i in xrange(50):
            bridges = d.getBridgesForIP(randomIP4String(),
                                        "faketimestamp", 1,
                                        bridgeFilterRules=[
                                            filterBridgesByIP4,
                                            filterBridgesByIP6])
            if bridges:
                t = bridges.pop()
                assert filterBridgesByIP4(t)
                assert filterBridgesByIP6(t)
                address, portlist = networkstatus.parseALine(
                    t.getConfigLine(addressClass=ipaddr.IPv4Address))
                assert type(address) is ipaddr.IPv4Address
                address, portlist = networkstatus.parseALine(
                    t.getConfigLine(addressClass=ipaddr.IPv6Address))
                assert type(address) is ipaddr.IPv6Address


    def testDistWithFilterAll(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
        for _ in xrange(250):
            d.insert(fakeBridge6(or_addresses=True))
            d.insert(fakeBridge(or_addresses=True))

        for i in xrange(5):
            b = d.getBridgesForIP(randomIP4String(), "x", 1, bridgeFilterRules=[
                filterBridgesByIP4, filterBridgesByIP6])
            assert len(b) == 0

    def testDistWithFilterBlockedCountries(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
        for _ in xrange(250):
            d.insert(fakeBridge6(or_addresses=True))
            d.insert(fakeBridge(or_addresses=True))

        for b in d.splitter.bridges:
            # china blocks all :-(
            for pt in b.transports:
                key = "%s:%s" % (pt.address, pt.port)
                b.blockingCountries[key] = set(['cn'])
            for address, portlist in b.or_addresses.items():
                for port in portlist:
                    key = "%s:%s" % (address, port)
                    b.blockingCountries[key] = set(['cn'])
            key = "%s:%s" % (b.ip, b.orport)
            b.blockingCountries[key] = set(['cn'])

        for i in xrange(5):
            b = d.getBridgesForIP(randomIP4String(), "x", 1, bridgeFilterRules=[
                filterBridgesByNotBlockedIn("cn")])
            assert len(b) == 0
            b = d.getBridgesForIP(randomIP4String(), "x", 1, bridgeFilterRules=[
                filterBridgesByNotBlockedIn("us")])
            assert len(b) > 0

    def testDistWithFilterBlockedCountriesAdvanced(self):
        d = bridgedb.Dist.IPBasedDistributor(self.dumbAreaMapper, 3, "Foo")
        for _ in xrange(250):
            d.insert(fakeBridge6(or_addresses=True, transports=True))
            d.insert(fakeBridge(or_addresses=True, transports=True))

        for b in d.splitter.bridges:
            # china blocks some transports
            for pt in b.transports:
                if random.choice(xrange(2)) > 0:
                    key = "%s:%s" % (pt.address, pt.port)
                    b.blockingCountries[key] = set(['cn'])
            for address, portlist in b.or_addresses.items():
                # china blocks some transports
                for port in portlist:
                    if random.choice(xrange(2)) > 0:
                        key = "%s:%s" % (address, port)
                        b.blockingCountries[key] = set(['cn'])
            key = "%s:%s" % (b.ip, b.orport)
            b.blockingCountries[key] = set(['cn'])

        # we probably will get at least one bridge back!
        # it's pretty unlikely to lose a coin flip 250 times in a row
        for i in xrange(5):
            b = d.getBridgesForIP(randomIPString(), "x", 1,
                    bridgeFilterRules=[
                        filterBridgesByNotBlockedIn("cn", methodname='obfs2'),
                        filterBridgesByTransport('obfs2'),
                        ])
            try: assert len(b) > 0
            except AssertionError:
                print("epic fail")
            b = d.getBridgesForIP(randomIPString(), "x", 1, bridgeFilterRules=[
                filterBridgesByNotBlockedIn("us")])
            assert len(b) > 0


class SQLStorageTests(unittest.TestCase):
    def setUp(self):
        self.fd, self.fname = tempfile.mkstemp()
        self.db = bridgedb.Storage.Database(self.fname)
        self.cur = self.db._conn.cursor()

    def tearDown(self):
        self.db.close()
        os.close(self.fd)
        os.unlink(self.fname)

    def assertCloseTo(self, a, b, delta=60):
        self.assertTrue(abs(a-b) <= delta)

    def testBridgeStorage(self):
        db = self.db
        B = bridgedb.Bridges.Bridge
        t = time.time()
        cur = self.cur

        k1 = "AAAAAAAAAAAAAAAAAAAABBBBBBBBBBBBBBBBBBBB"
        k2 = "ABABABABABABABABABABABABABABABABABABABAB"
        k3 = "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC"
        b1 = B("serv1", "1.2.3.4", 999, fingerprint=k1)
        b1_v2 = B("serv1", "1.2.3.5", 9099, fingerprint=k1)
        b2 = B("serv2", "2.3.4.5", 9990, fingerprint=k2)
        b3 = B("serv3", "2.3.4.6", 9008, fingerprint=k3)
        validRings = ["ring1", "ring2", "ring3"]

        r = db.insertBridgeAndGetRing(b1, "ring1", t, validRings)
        self.assertEquals(r, "ring1")
        r = db.insertBridgeAndGetRing(b1, "ring10", t+500, validRings)
        self.assertEquals(r, "ring1")

        cur.execute("SELECT distributor, address, or_port, first_seen, "
                    "last_seen FROM Bridges WHERE hex_key = ?", (k1,))
        v = cur.fetchone()
        self.assertEquals(v,
                          ("ring1", "1.2.3.4", 999,
                           bridgedb.Storage.timeToStr(t),
                           bridgedb.Storage.timeToStr(t+500)))

        r = db.insertBridgeAndGetRing(b1_v2, "ring99", t+800, validRings)
        self.assertEquals(r, "ring1")
        cur.execute("SELECT distributor, address, or_port, first_seen, "
                    "last_seen FROM Bridges WHERE hex_key = ?", (k1,))
        v = cur.fetchone()
        self.assertEquals(v,
                          ("ring1", "1.2.3.5", 9099,
                           bridgedb.Storage.timeToStr(t),
                           bridgedb.Storage.timeToStr(t+800)))

        db.insertBridgeAndGetRing(b2, "ring2", t, validRings)
        db.insertBridgeAndGetRing(b3, "ring3", t, validRings)

        cur.execute("SELECT COUNT(distributor) FROM Bridges")
        v = cur.fetchone()
        self.assertEquals(v, (3,))

        r = db.getEmailTime("abc@example.com")
        self.assertEquals(r, None)
        db.setEmailTime("abc@example.com", t)
        db.setEmailTime("def@example.com", t+1000)
        r = db.getEmailTime("abc@example.com")
        self.assertCloseTo(r, t)
        r = db.getEmailTime("def@example.com")
        self.assertCloseTo(r, t+1000)
        r = db.getEmailTime("ghi@example.com")
        self.assertEquals(r, None)

        db.cleanEmailedBridges(t+200)
        db.setEmailTime("def@example.com", t+5000)
        r = db.getEmailTime("abc@example.com")
        self.assertEquals(r, None)
        r = db.getEmailTime("def@example.com")
        self.assertCloseTo(r, t+5000)
        cur.execute("SELECT * FROM EmailedBridges")
        self.assertEquals(len(cur.fetchall()), 1)

        db.addBridgeBlock(b2.fingerprint, 'us')
        self.assertEquals(db.isBlocked(b2.fingerprint, 'us'), True)
        db.delBridgeBlock(b2.fingerprint, 'us')
        self.assertEquals(db.isBlocked(b2.fingerprint, 'us'), False)
        db.addBridgeBlock(b2.fingerprint, 'uk')
        db.addBridgeBlock(b3.fingerprint, 'uk')
        self.assertEquals(set([b2.fingerprint, b3.fingerprint]),
                set(db.getBlockedBridges('uk')))

        db.addBridgeBlock(b2.fingerprint, 'cn')
        db.addBridgeBlock(b2.fingerprint, 'de')
        db.addBridgeBlock(b2.fingerprint, 'jp')
        db.addBridgeBlock(b2.fingerprint, 'se')
        db.addBridgeBlock(b2.fingerprint, 'kr')

        self.assertEquals(set(db.getBlockingCountries(b2.fingerprint)),
                set(['uk', 'cn', 'de', 'jp', 'se', 'kr']))
        self.assertEquals(db.getWarnedEmail("def@example.com"), False)
        db.setWarnedEmail("def@example.com")
        self.assertEquals(db.getWarnedEmail("def@example.com"), True)
        db.setWarnedEmail("def@example.com", False)
        self.assertEquals(db.getWarnedEmail("def@example.com"), False)

        db.setWarnedEmail("def@example.com")
        self.assertEquals(db.getWarnedEmail("def@example.com"), True)
        db.cleanWarnedEmails(t+200)
        self.assertEquals(db.getWarnedEmail("def@example.com"), False)

class ParseCountryBlockFileTests(unittest.TestCase):

    def testParseCountryBlockFile(self):
        simpleBlock = "%s:%s %s\n"
        countries = ['us', 'nl', 'de', 'cz', 'sk', 'as', 'si', 'it']
        test = str()
        for i in range(100):
            test += simpleBlock % (randomIPString(), randomPort(),
                    randomCountrySpec())
            test+=gettimestamp()

        for a,p,c in bridgedb.Bridges.parseCountryBlockFile(test.split('\n')):
            assert type(a) in (ipaddr.IPv6Address, ipaddr.IPv4Address)
            assert isinstance(p, addr.PortList)
            assert isinstance(c, list)
            assert len(c) > 0
            for y in c:
                assert y in countries
            #print "address: %s" % a
            #print "portlist: %s" % p
            #print "countries: %s" % c

class BridgeStabilityTests(unittest.TestCase):
    def setUp(self):
        self.fd, self.fname = tempfile.mkstemp()
        self.db = bridgedb.Storage.Database(self.fname)
        bridgedb.Storage.setDB(self.db)
        self.cur = self.db._conn.cursor()

    def tearDown(self):
        self.db.close()
        os.close(self.fd)
        os.unlink(self.fname)

    def testAddOrUpdateSingleBridgeHistory(self):
        db = self.db
        b = fakeBridge()
        timestamp = time.time()
        bhe = bridgedb.Stability.addOrUpdateBridgeHistory(b, timestamp)
        assert isinstance(bhe, BridgeHistory)
        assert isinstance(db.getBridgeHistory(b.fingerprint), BridgeHistory)
        assert len([y for y in db.getAllBridgeHistory()]) == 1

    def testDeletingSingleBridgeHistory(self):
        db = self.db
        b = fakeBridge()
        timestamp = time.time()
        bhe = bridgedb.Stability.addOrUpdateBridgeHistory(b, timestamp)
        assert isinstance(bhe, BridgeHistory)
        assert isinstance(db.getBridgeHistory(b.fingerprint), BridgeHistory)
        db.delBridgeHistory(b.fingerprint)
        assert db.getBridgeHistory(b.fingerprint) is None
        assert len([y for y in db.getAllBridgeHistory()]) == 0

    def testTOSA(self):
        db = self.db
        b = random.choice([fakeBridge,fakeBridge6])()
        def timestampSeries(x):
            for i in xrange(61):
                yield (i+1)*60*30 + x # 30 minute intervals
        now = time.time()
        time_on_address = long(60*30*60) # 30 hours
        downtime = 60*60*random.randint(0,4) # random hours of downtime

        for t in timestampSeries(now):
            bridgedb.Stability.addOrUpdateBridgeHistory(b,t)
        assert db.getBridgeHistory(b.fingerprint).tosa == time_on_address

        b.orport += 1

        for t in timestampSeries(now + time_on_address + downtime):
            bhe = bridgedb.Stability.addOrUpdateBridgeHistory(b,t)
        assert db.getBridgeHistory(b.fingerprint).tosa == time_on_address + downtime

    def testLastSeenWithDifferentAddressAndPort(self):
        db = self.db
        for i in xrange(10):
            num_desc = 30
            time_start = time.time()
            ts = [ 60*30*(i+1) + time_start for i in xrange(num_desc) ]
            b = random.choice([fakeBridge(), fakeBridge6()])
            [ bridgedb.Stability.addOrUpdateBridgeHistory(b, t) for t in ts ]

            # change the port
            b.orport = b.orport+1
            last_seen = ts[-1]
            ts = [ 60*30*(i+1) + last_seen for i in xrange(num_desc) ]
            [ bridgedb.Stability.addOrUpdateBridgeHistory(b, t) for t in ts ]
            b = db.getBridgeHistory(b.fingerprint)
            assert b.tosa == ts[-1] - last_seen
            assert (long(last_seen*1000) == b.lastSeenWithDifferentAddressAndPort)
            assert (long(ts[-1]*1000) == b.lastSeenWithThisAddressAndPort)

    def testFamiliar(self):
        # create some bridges
        # XXX: slow
        num_bridges = 10
        num_desc = 4*48 # 30m intervals, 48 per day
        time_start = time.time()
        bridges = [ fakeBridge() for x in xrange(num_bridges) ]
        t = time.time()
        ts = [ (i+1)*60*30+t for i in xrange(num_bridges) ]
        for b in bridges:
            time_series = [ 60*30*(i+1) + time_start for i in xrange(num_desc) ]
            [ bridgedb.Stability.addOrUpdateBridgeHistory(b, i) for i in time_series ]
        assert None not in bridges
        # +1 to avoid rounding errors
        assert bridges[-(num_bridges/8 + 1)].familiar == True

    def testDiscountAndPruneBridgeHistory(self):
        """ Test pruning of old Bridge History """
        if os.environ.get('TRAVIS'):
            self.skipTest("Hangs on Travis-CI.")

        db = self.db

        # make a bunch of bridges
        num_bridges = 20
        time_start = time.time()
        bridges = [random.choice([fakeBridge, fakeBridge6])()
                   for i in xrange(num_bridges)]

        # run some of the bridges for the full time series
        running = bridges[:num_bridges/2]
        # and some that are not
        expired = bridges[num_bridges/2:]

        for b in running: assert b not in expired

        # Solving:
        # 1 discount event per 12 hours, 24 descriptors 30m apart
        num_successful = random.randint(2,60)
        # figure out how many intervals it will take for weightedUptime to
        # decay to < 1
        num_desc = int(30*log(1/float(num_successful*30*60))/(-0.05))
        timeseries = [ 60*30*(i+1) + time_start for i in xrange(num_desc) ]

        for i in timeseries:
            for b in running:
                bridgedb.Stability.addOrUpdateBridgeHistory(b, i)

            if num_successful > 0:
                for b in expired:
                    bridgedb.Stability.addOrUpdateBridgeHistory(b, i)
            num_successful -= 1

        # now we expect to see the bridge has been removed from history
        for bridge in expired:
            b = db.getBridgeHistory(bridge.fingerprint)
            assert b is None
        # and make sure none of the others have
        for bridge in running:
            b = db.getBridgeHistory(bridge.fingerprint)
            assert b is not None

def testSuite():
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()

    for klass in [IPBridgeDistTests, SQLStorageTests, EmailBridgeDistTests,
                  ParseCountryBlockFileTests, BridgeStabilityTests]:
        suite.addTest(loader.loadTestsFromTestCase(klass))

    for module in [ bridgedb.Bridges,
                    bridgedb.Main,
                    bridgedb.Dist,
                    bridgedb.schedule ]:
        suite.addTest(doctest.DocTestSuite(module))

    return suite

def main():
    suppressWarnings()

    unittest.TextTestRunner(verbosity=1).run(testSuite())




# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.email.templates` module."""

from __future__ import print_function
from __future__ import unicode_literals

from io import StringIO
from gettext import NullTranslations

from twisted.mail.smtp import Address
from twisted.trial import unittest

from bridgedb.email import templates


class EmailTemplatesTests(unittest.TestCase):
    """Unittests for :func:`b.e.templates`."""

    def setUp(self):
        self.t = NullTranslations(StringIO(unicode('test')))
        self.client = Address('blackhole@torproject.org')
        self.answer = 'obfs3 1.1.1.1:1111\nobfs3 2.2.2.2:2222'
        # This is the fingerprint of BridgeDB's offline, certification-only
        # GnuPG key. It should be present in any responses to requests for our
        # public keys.
        self.offlineFingerprint = '7B78437015E63DF47BB1270ACBD97AA24E8E472E'

    def shouldIncludeCommands(self, text):
        self.assertSubstring('COMMANDs', text)

    def shouldIncludeInstructions(self, text):
        self.assertSubstring('Tor Browser', text)

    def shouldIncludeBridges(self, text):
        self.assertSubstring(self.answer, text)
        self.assertSubstring('Here are your bridges:', text)

    def shouldIncludeGreeting(self, text):
        self.assertSubstring('Hey, blackhole!', text)

    def shouldIncludeAutomationNotice(self, text):
        self.assertSubstring('automated message', text)

    def shouldIncludeKey(self, text):
        self.assertSubstring('-----BEGIN PGP PUBLIC KEY BLOCK-----', text)

    def shouldIncludeFooter(self, text):
        self.assertSubstring('rainbows, unicorns, and sparkles', text)

    def test_templates_addCommands(self):
        text = templates.addCommands(self.t)
        self.shouldIncludeCommands(text)

    def test_templates_addGreeting(self):
        text = templates.addGreeting(self.t, self.client.local)
        self.shouldIncludeGreeting(text)

    def test_templates_addGreeting_noClient(self):
        text = templates.addGreeting(self.t, None)
        self.assertSubstring('Hello, friend!', text)

    def test_templates_addGreeting_withWelcome(self):
        text = templates.addGreeting(self.t, self.client.local, welcome=True)
        self.shouldIncludeGreeting(text)
        self.assertSubstring('Welcome to BridgeDB!', text)

    def test_templates_addGreeting_trueClient(self):
        text = templates.addGreeting(self.t, True)
        self.assertSubstring('Hey', text)

    def test_templates_addGreeting_23Client(self):
        text = templates.addGreeting(self.t, 23)
        self.assertSubstring('Hey', text)

    def test_templates_addHowto(self):
        text = templates.addHowto(self.t)
        self.shouldIncludeInstructions(text)

    def test_templates_addBridgeAnswer(self):
        text = templates.addBridgeAnswer(self.t, self.answer)
        self.shouldIncludeBridges(text)

    def test_templates_addFooter(self):
        text = templates.addFooter(self.t, self.client)
        self.shouldIncludeFooter(text)

    def test_templates_buildAnswerMessage(self):
        text = templates.buildAnswerMessage(self.t, self.client, self.answer)
        self.assertSubstring(self.answer, text)
        self.shouldIncludeAutomationNotice(text)
        self.shouldIncludeCommands(text)
        self.shouldIncludeFooter(text)

    def test_templates_buildKeyMessage(self):
        text = templates.buildKeyMessage(self.t, self.client)
        self.assertSubstring(self.offlineFingerprint, text)

    def test_templates_buildWelcomeText(self):
        text = templates.buildWelcomeText(self.t, self.client)
        self.shouldIncludeGreeting(text)
        self.assertSubstring('Welcome to BridgeDB!', text)
        self.shouldIncludeCommands(text)
        self.shouldIncludeFooter(text)

    def test_templates_buildSpamWarning(self):
        text = templates.buildSpamWarning(self.t, self.client)
        self.shouldIncludeGreeting(text)
        self.shouldIncludeAutomationNotice(text)
        self.shouldIncludeFooter(text)



"""
This module provides functionality for tracking bridge stability metrics, using
the model introduced in [1] and implemented in [2].

[1] Karsten Loesing, An Analysis of Tor Bridge Stability. Technical Report.
The Tor Project, October 2011.
https://metrics.torproject.org/papers/bridge-stability-2011-10-31.pdf

[2] https://gitweb.torproject.org/metrics-tasks/task-4255/SimulateBridgeStability.java
"""
import logging
import bridgedb.Storage

# tunables 
weighting_factor = float(19)/float(20)
discountIntervalMillis = long(60*60*12*1000)

class BridgeHistory(object):
    """ Record Class that tracks a single Bridge
    The fields stored are:

    fingerprint, ip, port, weightedUptime, weightedTime, weightedRunLength,
    totalRunWeights, lastSeenWithDifferentAddressAndPort,
    lastSeenWithThisAddressAndPort, lastDiscountedHistoryValues.

    fingerprint         The Bridge Fingerprint (unicode)
    ip                  The Bridge IP (unicode)
    port                The Bridge orport (integer)

    weightedUptime      Weighted uptime in seconds (long int)
    weightedTime        Weighted time in seconds (long int)
    weightedRunLength   Weighted run length of previous addresses or ports in
                        seconds. (long int)
    totalRunWeights     Total run weights of previously used addresses or
                        ports. (float)

    lastSeenWithDifferentAddressAndPort
        Timestamp in milliseconds when this
        bridge was last seen with a different address or port. (long int)

    lastSeenWithThisAddressAndPort
        Timestamp in milliseconds when this bridge was last seen
        with this address and port. (long int)

    lastDiscountedHistoryValues:
        Timestamp in milliseconds when this bridge was last discounted. (long int)

    lastUpdatedWeightedTime:
        Timestamp in milliseconds when the weighted time was updated. (long int)
    """
    def __init__(self, fingerprint, ip, port,
            weightedUptime, weightedTime, weightedRunLength, totalRunWeights,
            lastSeenWithDifferentAddressAndPort, lastSeenWithThisAddressAndPort,
            lastDiscountedHistoryValues, lastUpdatedWeightedTime):
        self.fingerprint = fingerprint
        self.ip = ip 
        self.port = port
        self.weightedUptime = long(weightedUptime)
        self.weightedTime = long(weightedTime)
        self.weightedRunLength = long(weightedRunLength)
        self.totalRunWeights = float(totalRunWeights)
        self.lastSeenWithDifferentAddressAndPort = \
                long(lastSeenWithDifferentAddressAndPort)
        self.lastSeenWithThisAddressAndPort = long(lastSeenWithThisAddressAndPort)
        self.lastDiscountedHistoryValues = long(lastDiscountedHistoryValues)
        self.lastUpdatedWeightedTime = long(lastUpdatedWeightedTime)

    def discountWeightedFractionalUptimeAndWeightedTime(self, discountUntilMillis):
        """ discount weighted times """
        if self.lastDiscountedHistoryValues == 0:
            self.lastDiscountedHistoryValues = discountUntilMillis
        rounds = self.numDiscountRounds(discountUntilMillis)
        if rounds > 0:
            discount = lambda x: (weighting_factor**rounds)*x
            self.weightedUptime = discount(self.weightedUptime)
            self.weightedTime = discount(self.weightedTime)
            self.weightedRunLength = discount(self.weightedRunLength)
            self.totalRunWeights = discount(self.totalRunWeights)

            self.lastDiscountedHistoryValues += discountIntervalMillis * rounds
        return rounds

    def numDiscountRounds(self, discountUntilMillis):
        """ return the number of rounds of discounting needed to bring this
        history element current """
        result = discountUntilMillis - self.lastDiscountedHistoryValues
        result = int(result/discountIntervalMillis)
        return max(result,0)

    @property
    def weightedFractionalUptime(self):
        """Weighted Fractional Uptime"""
        if self.weightedTime <0.0001: return long(0)
        return long(10000) * self.weightedUptime / self.weightedTime

    @property
    def tosa(self):
        """the Time On Same Address (TOSA)"""
        return ( self.lastSeenWithThisAddressAndPort - \
                    self.lastSeenWithDifferentAddressAndPort ) / 1000

    @property
    def familiar(self):
        """
        A bridge is 'familiar' if 1/8 of all active bridges have appeared
        more recently than it, or if it has been around for a Weighted Time of 8 days.
        """
        # if this bridge has been around longer than 8 days
        if self.weightedTime >= long(8 * 24 * 60 * 60):
            return True

        # return True if self.weightedTime is greater than the weightedTime
        # of the > bottom 1/8 all bridges, sorted by weightedTime
        with bridgedb.Storage.getDB() as db:
            allWeightedTimes = [ bh.weightedTime for bh in db.getAllBridgeHistory()]
            numBridges = len(allWeightedTimes)
            logging.debug("Got %d weightedTimes", numBridges)
            allWeightedTimes.sort()
            if self.weightedTime >= allWeightedTimes[numBridges/8]:
                return True
            return False

    @property
    def wmtbac(self):
        """Weighted Mean Time Between Address Change"""
        totalRunLength = self.weightedRunLength + \
                ((self.lastSeenWithThisAddressAndPort -
                self.lastSeenWithDifferentAddressAndPort) / long(1000))

        totalWeights = self.totalRunWeights + 1.0
        if totalWeights <  0.0001: return long(0)
        assert(isinstance(long,totalRunLength))
        assert(isinstance(long,totalWeights))
        return totalRunlength / totalWeights

def addOrUpdateBridgeHistory(bridge, timestamp):
    with bridgedb.Storage.getDB() as db:
        bhe = db.getBridgeHistory(bridge.fingerprint)
        if not bhe:
            # This is the first status, assume 60 minutes.
            secondsSinceLastStatusPublication = long(60*60)
            lastSeenWithDifferentAddressAndPort = timestamp * long(1000)
            lastSeenWithThisAddressAndPort = timestamp * long(1000)
    
            bhe = BridgeHistory(
                    bridge.fingerprint, bridge.address, bridge.orPort,
                    0,#weightedUptime
                    0,#weightedTime
                    0,#weightedRunLength
                    0,# totalRunWeights
                    lastSeenWithDifferentAddressAndPort, # first timestamnp
                    lastSeenWithThisAddressAndPort,
                    0,#lastDiscountedHistoryValues,
                    0,#lastUpdatedWeightedTime
                    )
            # first time we have seen this descriptor
            db.updateIntoBridgeHistory(bhe)
        # Calculate the seconds since the last parsed status.  If this is
        # the first status or we haven't seen a status for more than 60
        # minutes, assume 60 minutes.
        statusPublicationMillis = long(timestamp * 1000)
        if (statusPublicationMillis - bhe.lastSeenWithThisAddressAndPort) > 60*60*1000:
            secondsSinceLastStatusPublication = long(60*60)
            logging.debug("Capping secondsSinceLastStatusPublication to 1 hour")
        # otherwise, roll with it
        else:
            secondsSinceLastStatusPublication = \
                    (statusPublicationMillis - bhe.lastSeenWithThisAddressAndPort)/1000
        if secondsSinceLastStatusPublication <= 0 and bhe.weightedTime > 0:
            # old descriptor, bail
            logging.warn("Received old descriptor for bridge %s with timestamp %d",
                    bhe.fingerprint, statusPublicationMillis/1000)
            return bhe
    
        # iterate over all known bridges and apply weighting factor
        discountAndPruneBridgeHistories(statusPublicationMillis)

        # Update the weighted times of bridges
        updateWeightedTime(statusPublicationMillis)

        # For Running Bridges only:
        # compare the stored history against the descriptor and see if the
        # bridge has changed its address or port
        bhe = db.getBridgeHistory(bridge.fingerprint)

        if not bridge.running:
            logging.info("%s is not running" % bridge.fingerprint)
            return bhe

        # Parse the descriptor and see if the address or port changed
        # If so, store the weighted run time
        if bridge.orport != bhe.port or bridge.ip != bhe.ip:
            bhe.totalRunWeights += 1.0;
            bhe.weightedRunLength += bhe.tosa
            bhe.lastSeenWithDifferentAddressAndPort =\
                    bhe.lastSeenWithThisAddressAndPort

        # Regardless of whether the bridge is new, kept or changed
        # its address and port, raise its WFU times and note its
        # current address and port, and that we saw it using them.
        bhe.weightedUptime += secondsSinceLastStatusPublication
        bhe.lastSeenWithThisAddressAndPort = statusPublicationMillis
        bhe.ip = str(bridge.ip)
        bhe.port = bridge.orport
        return db.updateIntoBridgeHistory(bhe)

def discountAndPruneBridgeHistories(discountUntilMillis):
    with bridgedb.Storage.getDB() as db:
        bhToRemove = []
        bhToUpdate = []

        for bh in db.getAllBridgeHistory():
            # discount previous values by factor of 0.95 every 12 hours
            bh.discountWeightedFractionalUptimeAndWeightedTime(discountUntilMillis)
            # give the thing at least 24 hours before pruning it
            if bh.weightedFractionalUptime < 1 and bh.weightedTime > 60*60*24:
                logging.debug("Removing bridge from history: %s" % bh.fingerprint)
                bhToRemove.append(bh.fingerprint)
            else:
                bhToUpdate.append(bh)

        for k in bhToUpdate: db.updateIntoBridgeHistory(k)
        for k in bhToRemove: db.delBridgeHistory(k)

def updateWeightedTime(statusPublicationMillis):
    bhToUpdate = []
    with bridgedb.Storage.getDB() as db:
        for bh in db.getBridgesLastUpdatedBefore(statusPublicationMillis):
            interval = (statusPublicationMillis - bh.lastUpdatedWeightedTime)/1000
            if interval > 0:
                bh.weightedTime += min(3600,interval) # cap to 1hr
                bh.lastUpdatedWeightedTime = statusPublicationMillis
                #db.updateIntoBridgeHistory(bh)
                bhToUpdate.append(bh)

        for bh in bhToUpdate:
            db.updateIntoBridgeHistory(bh)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: see LICENSE for licensing information


"""Helpers for testing the email distributor and its servers."""


import io

from bridgedb.Dist import IgnoreEmail
from bridgedb.Dist import TooSoonEmail
from bridgedb.persistent import Conf
from bridgedb.email.server import MailServerContext
from bridgedb.schedule import Unscheduled
from bridgedb.test.test_HTTPServer import DummyBridge


EMAIL_DIST = True
EMAIL_INCLUDE_FINGERPRINTS = True
EMAIL_GPG_SIGNING_ENABLED = True
EMAIL_GPG_HOMEDIR = '.gnupg'
EMAIL_GPG_PRIMARY_KEY_FINGERPRINT = '0017098C5DF4197E3C884DCFF1B240D43F148C21'
EMAIL_GPG_PASSPHRASE = None
EMAIL_GPG_PASSPHRASE_FILE = None
EMAIL_DOMAIN_MAP = {
   'googlemail.com': 'gmail.com',
   'mail.google.com': 'gmail.com',
}
EMAIL_DOMAIN_RULES = {
   'gmail.com': ["ignore_dots", "dkim"],
   'example.com': [],
   'localhost': [],
}
EMAIL_DOMAINS = ["gmail.com", "example.com", "localhost"]
EMAIL_WHITELIST = {'white@list.ed': 'ABCD1234ABCD1234ABCD1234ABCD1234ABCD1234'}
EMAIL_BLACKLIST = ['feidanchaoren0001@gmail.com']
EMAIL_FUZZY_MATCH = 4
EMAIL_USERNAME = "bridges"
EMAIL_SMTP_HOST = "127.0.0.1"
EMAIL_SMTP_PORT = 25
EMAIL_SMTP_FROM_ADDR = "bridges@localhost"
EMAIL_N_BRIDGES_PER_ANSWER = 3
EMAIL_FROM_ADDR = "bridges@localhost"
EMAIL_BIND_IP = "127.0.0.1"
EMAIL_PORT = 5225

TEST_CONFIG_FILE = io.StringIO(unicode("""\
EMAIL_DIST = %s
EMAIL_INCLUDE_FINGERPRINTS = %s
EMAIL_GPG_SIGNING_ENABLED = %s
EMAIL_GPG_HOMEDIR = %s
EMAIL_GPG_PRIMARY_KEY_FINGERPRINT = %s
EMAIL_GPG_PASSPHRASE = %s
EMAIL_GPG_PASSPHRASE_FILE = %s
EMAIL_DOMAIN_MAP = %s
EMAIL_DOMAIN_RULES = %s
EMAIL_DOMAINS = %s
EMAIL_WHITELIST = %s
EMAIL_BLACKLIST = %s
EMAIL_FUZZY_MATCH = %s
EMAIL_USERNAME = %s
EMAIL_SMTP_HOST = %s
EMAIL_SMTP_PORT = %s
EMAIL_SMTP_FROM_ADDR = %s
EMAIL_N_BRIDGES_PER_ANSWER = %s
EMAIL_FROM_ADDR = %s
EMAIL_BIND_IP = %s
EMAIL_PORT = %s
""" % (repr(EMAIL_DIST),
       repr(EMAIL_INCLUDE_FINGERPRINTS),
       repr(EMAIL_GPG_SIGNING_ENABLED),
       repr(EMAIL_GPG_HOMEDIR),
       repr(EMAIL_GPG_PRIMARY_KEY_FINGERPRINT),
       repr(EMAIL_GPG_PASSPHRASE),
       repr(EMAIL_GPG_PASSPHRASE_FILE),
       repr(EMAIL_DOMAIN_MAP),
       repr(EMAIL_DOMAIN_RULES),
       repr(EMAIL_DOMAINS),
       repr(EMAIL_WHITELIST),
       repr(EMAIL_BLACKLIST),
       repr(EMAIL_FUZZY_MATCH),
       repr(EMAIL_USERNAME),
       repr(EMAIL_SMTP_HOST),
       repr(EMAIL_SMTP_PORT),
       repr(EMAIL_SMTP_FROM_ADDR),
       repr(EMAIL_N_BRIDGES_PER_ANSWER),
       repr(EMAIL_FROM_ADDR),
       repr(EMAIL_BIND_IP),
       repr(EMAIL_PORT))))


def _createConfig(configFile=TEST_CONFIG_FILE):
    configuration = {}
    TEST_CONFIG_FILE.seek(0)
    compiled = compile(configFile.read(), '<string>', 'exec')
    exec compiled in configuration
    config = Conf(**configuration)
    return config

def _createMailServerContext(config=None, distributor=None):
    if not config:
        config = _createConfig()

    if not distributor:
        distributor = DummyEmailDistributor(
            domainmap=config.EMAIL_DOMAIN_MAP,
            domainrules=config.EMAIL_DOMAIN_RULES)

    context = MailServerContext(config, distributor, Unscheduled())
    return context


class DummyEmailDistributor(object):
    """A mocked :class:`bridgedb.Dist.EmailBasedDistributor` which is used to
    test :class:`bridgedb.EmailServer`.
    """

    def __init__(self, key=None, domainmap=None, domainrules=None,
                 answerParameters=None):
        """None of the parameters are really used, ― they are just there to retain an
        identical method signature.
        """
        self.key = self.__class__.__name__
        self.domainmap = domainmap
        self.domainrules = domainrules
        self.answerParameters = answerParameters

    def getBridgesForEmail(self, emailaddress, epoch, N=1, parameters=None,
                           countryCode=None, bridgeFilterRules=None):
        return [DummyBridge() for _ in xrange(N)]

    def cleanDatabase(self):
        pass


class DummyEmailDistributorWithState(DummyEmailDistributor):
    """A mocked :class:`bridgedb.Dist.EmailBasedDistributor` which raises
    :exc:`bridgedb.Dist.TooSoonEmail` on the second email and
    :exc:`bridgedb.Dist.IgnoreEmail` on the third.

    Note that the state tracking is done in a really dumb way. For example, we
    currently don't consider requests for help text or GnuPG keys to be a
    "real" request, so in the real email distributor they won't trigger either
    a TooSoonEmail or IgnoreEmail. Here we only track the total number of
    *any* type of request per client.
    """

    def __init__(self, *args, **kwargs):
        super(DummyEmailDistributorWithState, self).__init__()
        self.alreadySeen = {}

    def getBridgesForEmail(self, emailaddress, epoch, N=1, parameters=None,
                           countryCode=None, bridgeFilterRules=None):
        # Keep track of the number of times we've seen a client.
        if not emailaddress in self.alreadySeen.keys():
            self.alreadySeen[emailaddress] = 0
        self.alreadySeen[emailaddress] += 1

        if self.alreadySeen[emailaddress] <= 1:
            return [DummyBridge() for _ in xrange(N)]
        elif self.alreadySeen[emailaddress] == 2:
            raise TooSoonEmail(
                "Seen client '%s' %d times"
                % (emailaddress, self.alreadySeen[emailaddress]),
                emailaddress)
        else:
            raise IgnoreEmail(
                "Seen client '%s' %d times"
                % (emailaddress, self.alreadySeen[emailaddress]),
                emailaddress)



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_util -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           Matthew Finkel 0x017DD169EA793BE2 <sysrqb@torproject.org>
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2013-2015, Matthew Finkel
#             (c) 2007-2015, The Tor Project, Inc.
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Common utilities for BridgeDB."""

from functools import partial

import abc
import logging
import logging.config
import logging.handlers
import os


def _getLogHandlers(logToFile=True, logToStderr=True):
    """Get the appropriate list of log handlers.

    :param bool logToFile: If ``True``, add a logfile handler.
    :param bool logToStderr: If ``True``, add a stream handler to stderr.
    :rtype: list
    :returns: A list containing the appropriate log handler names from the
        :class:`logging.config.dictConfigClass`.
    """
    logHandlers = []
    if logToFile:
        logHandlers.append('rotating')
    if logToStderr:
        logHandlers.append('console')
    return logHandlers

def _getRotatingFileHandler(filename, mode='a', maxBytes=1000000, backupCount=0,
                            encoding='utf-8', uid=None, gid=None):
    """Get a :class:`logging.RotatingFileHandler` with a logfile which is
    readable+writable only by the given **uid** and **gid**.

    :param str filename: The full path to the log file.
    :param str mode: The mode to open **filename** with. (default: ``'a'``)
    :param int maxBytes: Rotate logfiles after they have grown to this size in
        bytes.
    :param int backupCount: The number of logfiles to keep in rotation.
    :param str encoding: The encoding for the logfile.
    :param int uid: The owner UID to set on the logfile.
    :param int gid: The GID to set on the logfile.
    :rtype: :class:`logging.handlers.RotatingFileHandler`
    :returns: A logfile handler which will rotate files and chown/chmod newly
        created files.
    """
    # Default to the current process owner's uid and gid:
    uid = os.getuid() if not uid else uid
    gid = os.getgid() if not gid else gid

    if not os.path.exists(filename):
        open(filename, 'a').close()
    os.chown(filename, uid, gid)
    try:
        os.chmod(filename, os.ST_WRITE | os.ST_APPEND)
    except AttributeError:  # pragma: no cover
        logging.error("""
    XXX FIXME: Travis chokes on `os.ST_WRITE` saying that the module doesn't
               have that attribute, for some reason:
    https://travis-ci.org/isislovecruft/bridgedb/builds/24145963#L1601""")
        os.chmod(filename, 384)

    fileHandler = partial(logging.handlers.RotatingFileHandler,
                          filename,
                          mode,
                          maxBytes=maxBytes,
                          backupCount=backupCount,
                          encoding=encoding)
    return fileHandler

def configureLogging(cfg):
    """Set up Python's logging subsystem based on the configuration.

    :type cfg: :class:`~bridgedb.persistent.Conf`
    :param cfg: The current configuration, including any in-memory settings.
    """
    from bridgedb import safelog

    # Turn on safe logging by default:
    safelogging = getattr(cfg, 'SAFELOGGING', True)
    safelog.setSafeLogging(safelogging)

    level = getattr(cfg, 'LOGLEVEL', 'WARNING')
    logLevel = getattr(logging, level, 0)
    logStderr = getattr(cfg, 'LOG_TO_STDERR', False)
    logfileName = getattr(cfg, 'LOGFILE', "bridgedb.log")
    logfileCount = getattr(cfg, 'LOGFILE_COUNT', 3) - 1
    logfileRotateSize = getattr(cfg, 'LOGFILE_ROTATE_SIZE', 10000000)
    logThreads = getattr(cfg, 'LOG_THREADS', False)
    logTrace = getattr(cfg, 'LOG_TRACE', False)
    logTimeFormat = getattr(cfg, 'LOG_TIME_FORMAT', "%H:%M:%S")

    logFilters = []
    if safelogging:
        logFilters = ['safelogEmail', 'safelogIPv4', 'safelogIPv6']

    logConfig = {
        'version': 1,
        'filters': {
            'safelogEmail': {'()': safelog.SafelogEmailFilter},
            'safelogIPv4': {'()': safelog.SafelogIPv4Filter},
            'safelogIPv6': {'()': safelog.SafelogIPv6Filter},
        },
        'formatters': {
            'default': {'()': JustifiedLogFormatter,
                        # These values below are kwargs passed to
                        # :class:`JustifiedFormatter`:
                        'logThreads': logThreads,
                        'logTrace': logTrace,
                        'datefmt': logTimeFormat},
        },
        'handlers': {
            'console': {'class': 'logging.StreamHandler',
                        'level': logLevel,
                        'formatter': 'default',
                        'filters': logFilters},
            'rotating': {'()': _getRotatingFileHandler(logfileName, 'a',
                                                       logfileRotateSize,
                                                       logfileCount),
                         'level': logLevel,
                         'formatter': 'default',
                         'filters': logFilters},
        },
        'root': {
            'handlers': _getLogHandlers(logfileName, logStderr),
            'level': logLevel,
        },
    }

    logging.config.dictConfig(logConfig)

    logging.info("Logger Started.")
    logging.info("Level: %s", logLevel)
    logging.info("Safe Logging: %sabled" % ("En" if safelogging else "Dis"))

def levenshteinDistance(s1, s2, len1=None, len2=None,
                        offset1=0, offset2=0, memo=None):
    """Compute the Levenstein Distance between two strings.

    The `Levenshtein String Distance Algorithm
    <https://en.wikipedia.org/wiki/Levenshtein_distance>` efficiently computes
    the number of characters which must be changed in **s1** to make it
    identical to **s2**.

    >>> levenshteinDistance('cat', 'cat')
    0
    >>> levenshteinDistance('cat', 'hat')
    1
    >>> levenshteinDistance('arma', 'armadillo')
    5

    :param str s1: The string which should be changed.
    :param str s2: The string which **stringOne** should be compared to.
    """
    len1 = len(s1) if len1 is None else len1
    len2 = len(s2) if len2 is None else len2
    memo = {} if memo is None else memo

    key = ','.join([str(offset1), str(len1), str(offset2), str(len2)])
    if memo.get(key) is not None: return memo[key]

    if len1 == 0: return len2
    elif len2 == 0: return len1

    cost = 0 if (s1[offset1] == s2[offset2]) else 1
    distance = min(
        levenshteinDistance(s1, s2, len1-1, len2,   offset1+1, offset2,   memo) + 1,
        levenshteinDistance(s1, s2, len1,   len2-1, offset1,   offset2+1, memo) + 1,
        levenshteinDistance(s1, s2, len1-1, len2-1, offset1+1, offset2+1, memo) + cost,
    )
    memo[key] = distance
    return distance


class JustifiedLogFormatter(logging.Formatter):
    """A logging formatter which pretty prints thread and calling function
    information, in addition to the normal timestamp, log level, and log
    message.

    :ivar int width: The width of the column for the calling function
        information, if the latter is to be included.
    """
    width = 30

    def __init__(self, logThreads=False, logTrace=False,
                 datefmt="%H:%M:%s"):
        """If **logTrace** is ``True``, the line number, module name, and
        function name where the logger was called will be included in the
        message, and the width of this information will always equal ``width``.

        :param bool logThreads: If ``True``, include the current thread name
            and ID in formatted log messages.
        :param bool logTrace: If ``True``, include information on the calling
            function in formatted log messages.
        """
        super(JustifiedLogFormatter, self).__init__(datefmt=datefmt)
        self.logThreads = logThreads
        self.logTrace = logTrace

        _fmt = ["%(asctime)s %(levelname)-7.7s"]
        if self.logThreads:
            _fmt.append("[%(threadName)s id:%(thread)d]")
        _fmt.append("%(callingFunc)s")
        _fmt.append("%(message)s")

        self._fmt = " ".join(_fmt)

    def _formatCallingFuncName(self, record):
        """Format the combined module name and function name of the place where
        the log message/record was recorded, so that the formatted string is
        left-justified and not longer than the :cvar:`width`.

        :type record: :class:`logging.LogRecord`
        :param record: A record of an event created by calling a logger.
        :returns: The :class:`logging.LogRecord` with its ``message``
            attribute rewritten to contain the module and function name,
            truncated to ``width``, or padded on the right with spaces as is
            necessary.
        """
        callingFunc = ""
        if self.logTrace:
            # The '.' character between the module name and function name
            # would otherwise be interpreted as a format string specifier, so
            # we must specify ``chr(46)``:
            lineno = "L%s:" % record.lineno
            caller = "%s%-s%s" % (lineno.rjust(6), record.module, chr(46))
            maxFuncNameWidth = self.width - 2 - len(caller)
            funcName = record.funcName
            if len(funcName) > maxFuncNameWidth:
                funcName = record.funcName[:maxFuncNameWidth]
            caller += "%s()" % (funcName)
            callingFunc = caller.ljust(self.width)

        record.callingFunc = callingFunc
        return record

    def format(self, record):
        """Reformat this log **record** to neatly print thread and function
        traces, if configured to do so.

        :type record: :class:`logging.LogRecord`
        :param record: A record of an event created by calling a logger.
        """
        record = self._formatCallingFuncName(record)
        return super(JustifiedLogFormatter, self).format(record)


class mixin:
    """Subclasses of me can be used as a mixin class by registering another
    class, ``ClassA``, which should be mixed with the ``mixin`` subclass, in
    order to provide simple, less error-prone, multiple inheritance models::

    >>> from __future__ import print_function
    >>> from bridgedb.util import mixin
    >>>
    >>> class ClassA(object):
    >>>     def sayWhich(self):
    >>>         print("ClassA.sayWhich() called.")
    >>>     def doSuperThing(self):
    >>>         super(ClassA, self).__repr__()
    >>>     def doThing(self):
    >>>         print("ClassA is doing a thing.")
    >>>
    >>> class ClassB(ClassA):
    >>>     def sayWhich(self):
    >>>         print("ClassB.sayWhich() called.")
    >>>     def doSuperThing(self):
    >>>         super(ClassB, self).__repr__()
    >>>     def doOtherThing(self):
    >>>         print("ClassB is doing something else.")
    >>>
    >>> class ClassM(mixin):
    >>>     def sayWhich(self):
    >>>         print("ClassM.sayWhich() called.")
    >>>
    >>> ClassM.register(ClassA)
    >>>
    >>> class ClassC(ClassM, ClassB):
    >>>     def sayWhich(self):
    >>>         super(ClassC, self).sayWhich()
    >>>
    >>> c = ClassC()
    >>> c.sayWhich()
    ClassM.saywhich() called.
    >>> c.doSuperThing()
    <super: <class 'ClassA'>, NULL>
    >>> c.doThing()
    ClassA is doing a thing.
    >>> c.doOtherThing()
    ClassB is doing something else.

    .. info:: This class' name is lowercased because pylint is hardcoded to
        expect mixin classes to end in ``'mixin'``.
    """
    __metaclass__ = abc.ABCMeta



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_parse_fingerprint ; -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2014-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________


import binascii
import logging


#: The required length for hexidecimal representations of hash digest of a
#: Tor relay's public identity key (a.k.a. its fingerprint).
HEX_FINGERPRINT_LEN = 40


#: (callable) Convert a value from binary to hexidecimal representation.
toHex = binascii.b2a_hex

#: (callable) Convert a value from hexidecimal to binary representation.
fromHex = binascii.a2b_hex

def isValidFingerprint(fingerprint):
    """Determine if a Tor relay fingerprint is valid.

    :param str fingerprint: The hex-encoded hash digest of the relay's
        public identity key, a.k.a. its fingerprint.
    :rtype: bool
    :returns: ``True`` if the **fingerprint** was valid, ``False`` otherwise.
    """
    try:
        if len(fingerprint) != HEX_FINGERPRINT_LEN:
            raise ValueError("Fingerprint has incorrect length: %r"
                             % repr(fingerprint))
        fromHex(fingerprint)
    except (TypeError, ValueError):
        logging.debug("Invalid hex fingerprint: %r" % repr(fingerprint))
    else:
        return True
    return False



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for :class:`bridgedb.parse.descriptors` module."""

from __future__ import print_function

import datetime
import glob
import io
import os
import textwrap

from twisted.trial import unittest

HAS_STEM = False

try:
    from stem.descriptor.server_descriptor import RelayDescriptor
    from stem.descriptor.extrainfo_descriptor import RelayExtraInfoDescriptor
    from stem.descriptor.router_status_entry import RouterStatusEntryV3
    from bridgedb.parse import descriptors
except (ImportError, NameError), error:
    print("There was an error importing stem: %s" % error)
else:
    HAS_STEM = True


BRIDGE_NETWORKSTATUS_0 = '''\
r MiserLandfalls 4IsyTSCtChPhFPAnq5rD8yymlqA /GMC4lz8RXT/62v6kZNdmzSmopk 2014-11-04 06:23:22 2.215.61.223 4056 0
a [c5fd:4467:98a7:90be:c76a:b449:8e6f:f0a7]:4055
s Fast Guard Running Stable Valid
w Bandwidth=1678904
p reject 1-65535
'''

BRIDGE_NETWORKSTATUS_1 = '''\
r Unmentionable BgOrX0ViP5hNsK5ZvixAuPZ6EY0 NTg9NoE5ls9KjF96Dp/UdrabZ9Y 2014-11-04 12:23:37 80.44.173.87 51691 0
a [da14:7d1e:ba8e:60d0:b078:3f88:382b:5c70]:51690
s Fast Guard Running Stable Valid
w Bandwidth=24361
p reject 1-65535
'''

BRIDGE_SERVER_DESCRIPTOR = '''\
@purpose bridge
router MiserLandfalls 2.215.61.223 4056 0 0
or-address [c5fd:4467:98a7:90be:c76a:b449:8e6f:f0a7]:4055
platform Tor 0.2.2.39 on Linux
opt protocols Link 1 2 Circuit 1
published 2014-11-04 06:23:22
opt fingerprint E08B 324D 20AD 0A13 E114 F027 AB9A C3F3 2CA6 96A0
uptime 24247659
bandwidth 1977077890 2234957615 1719198165
opt extra-info-digest 1CBBB3D6158F324476E6804B7EE25623899271CB
onion-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBAOm4NX2wi8JmgcAyvOyiAEfq9UkzaNHK+VnSZBiPIrb5GAKFibR7S+Bb
7+x7tsT8VBNbe9QmwML2GVah3xXg68gJAksMNIgFdpud+zMhduuGd0jr7V55aLmH
ePGJYCh78B9RqfvmeTridp3pljwcAheKKH/YKi3nv1fPY0BwahurAgMBAAE=
-----END RSA PUBLIC KEY-----
signing-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBANd/JkrTZRT24EkK3DDc/E+Nj1QBnKIm/xXMyW0gkotFOVdewIWjwQ5z
Tn3YbDhrFN0aFYVdVwNbRhW83e+jZDkpIQuxlQOx6bT13vrzmg8ff1tH8I9EePl7
MO4v0DLPIEcu7Zfz90oC1bl36oqNsD4h0v4yK/XjVwLutIGiy3gTAgMBAAE=
-----END RSA PUBLIC KEY-----
contact Somebody <somebody@example.com>
ntor-onion-key NBsk2O6ks5qnxLhhhKPd59zi0IzfjnakoOJP+Cm8OAE
reject *:*
router-signature
-----BEGIN SIGNATURE-----
YYA5wJTHcjqXk/QBaDXHX/4Fb8W2OctF4X4VHyxH9Hsou4Ip7nzdfWzbBTcBiIrt
ybaaMO15L9Ctkli/capN+nCw2jWgivgiPnAmJNmLGeN6skTKjLPAau+839hBuQxu
P2aB/+XQfzFBA5TaWF83coDng4OGodhwHaOx10Kn7Bg=
-----END SIGNATURE-----
'''

BRIDGE_EXTRA_INFO_DESCRIPTOR = '''\
extra-info MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0
published 2014-11-04 06:23:22
write-history 2014-11-04 06:23:22 (900 s) 3188736,2226176,2866176
read-history 2014-11-04 06:23:22 (900 s) 3891200,2483200,2698240
dirreq-write-history 2014-11-04 06:23:22 (900 s) 1024,0,2048
dirreq-read-history 2014-11-04 06:23:22 (900 s) 0,0,0
geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F
geoip6-db-digest E983833985E4BCA34CEF611B2DF51942D188E638
dirreq-stats-end 2014-11-04 06:23:22 (86400 s)
dirreq-v3-ips
dirreq-v3-reqs
dirreq-v3-resp ok=16,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=12,timeout=0,running=0
transport obfs3 2.215.61.223:4057
transport obfs2 2.215.61.223:4058
transport scramblesuit 2.215.61.223:4059 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
transport obfs4 2.215.61.223:4060 iat-mode=0,node-id=19a448c01aa2e7d55979473b647e282459995b85,public-key=7a61b53701befdae0eeeffaecc73f14e20b537bb0f8b91ad7c2936dc63562b25
bridge-stats-end 2014-11-04 06:23:22 (86400 s)
bridge-ips ca=8
bridge-ip-versions v4=8,v6=0
bridge-ip-transports <OR>=8
router-signature
-----BEGIN SIGNATURE-----
KOXNPCoe+Q+thFA/Lz7RTja2tWp4oC6SvyIooEZibHtEDgiXuU4sELWT4bSOk3np
RVmu7QPMmNybx4LHowq3pOeNLtJzpWg8Pfo+N6tR+K4nqPwBRmpsuDhCD/tIXJlP
U36EY4UoN5ABPowhNZFeyr5A3vKiDr6j0hCOqYOhxPY=
-----END SIGNATURE-----
'''

BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE = '''\
extra-info MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0
published 2014-11-04 08:10:25
write-history 2014-11-04 08:10:25 (900 s) 3188736,2226176,2866176,2226176
read-history 2014-11-04 08:10:25 (900 s) 3891200,2483200,2698240,2483200
dirreq-write-history 2014-11-04 08:10:25 (900 s) 1024,0,2048,3072
dirreq-read-history 2014-11-04 08:10:25 (900 s) 0,0,0,0
geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F
geoip6-db-digest E983833985E4BCA34CEF611B2DF51942D188E638
dirreq-stats-end 2014-11-04 08:10:25 (86400 s)
dirreq-v3-ips
dirreq-v3-reqs
dirreq-v3-resp ok=16,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=12,timeout=0,running=0
transport obfs3 2.215.61.223:4057
transport obfs2 2.215.61.223:4058
transport scramblesuit 2.215.61.223:4059 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
transport obfs4 2.215.61.223:4060 iat-mode=0,node-id=19a448c01aa2e7d55979473b647e282459995b85,public-key=7a61b53701befdae0eeeffaecc73f14e20b537bb0f8b91ad7c2936dc63562b25
bridge-stats-end 2014-11-04 08:10:25 (86400 s)
bridge-ips ca=8
bridge-ip-versions v4=8,v6=0
bridge-ip-transports <OR>=8
router-signature
-----BEGIN SIGNATURE-----
KOXNPCoe+Q+thFA/Lz7RTja2tWp4oC6SvyIooEZibHtEDgiXuU4sELWT4bSOk3np
RVmu7QPMmNybx4LHowq3pOeNLtJzpWg8Pfo+N6tR+K4nqPwBRmpsuDhCD/tIXJlP
U36EY4UoN5ABPowhNZFeyr5A3vKiDr6j0hCOqYOhxPY=
-----END SIGNATURE-----
'''

BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE = '''\
extra-info MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0
published 2014-12-04 03:10:25
write-history 2014-12-04 03:10:25 (900 s) 3188736,2226176,2866176,2226176
read-history 2014-12-04 03:10:25 (900 s) 3891200,2483200,2698240,2483200
dirreq-write-history 2014-12-04 03:10:25 (900 s) 1024,0,2048,3072
dirreq-read-history 2014-12-04 03:10:25 (900 s) 0,0,0,0
geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F
geoip6-db-digest E983833985E4BCA34CEF611B2DF51942D188E638
dirreq-stats-end 2014-12-04 03:10:25 (86400 s)
dirreq-v3-ips
dirreq-v3-reqs
dirreq-v3-resp ok=16,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=12,timeout=0,running=0
transport obfs3 2.215.61.223:4057
transport obfs2 2.215.61.223:4058
transport scramblesuit 2.215.61.223:4059 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
transport obfs4 2.215.61.223:4060 iat-mode=0,node-id=19a448c01aa2e7d55979473b647e282459995b85,public-key=7a61b53701befdae0eeeffaecc73f14e20b537bb0f8b91ad7c2936dc63562b25
bridge-stats-end 2014-12-04 03:10:25 (86400 s)
bridge-ips ca=8
bridge-ip-versions v4=8,v6=0
bridge-ip-transports <OR>=8
router-signature
-----BEGIN SIGNATURE-----
KOXNPCoe+Q+thFA/Lz7RTja2tWp4oC6SvyIooEZibHtEDgiXuU4sELWT4bSOk3np
RVmu7QPMmNybx4LHowq3pOeNLtJzpWg8Pfo+N6tR+K4nqPwBRmpsuDhCD/tIXJlP
U36EY4UoN5ABPowhNZFeyr5A3vKiDr6j0hCOqYOhxPY=
-----END SIGNATURE-----
'''


class ParseDescriptorsTests(unittest.TestCase):
    """Unittests for :class:`bridgedb.parse.descriptors` module."""

    skip = True if not HAS_STEM else False

    def setUp(self):
        """Test if we have Stem installed. Skip these tests if it's missing."""
        self.expectedIPBridge0 = '2.215.61.223'
        self.expectedIPBridge1 = '80.44.173.87'

        self.expectedFprBridge0 = 'E08B324D20AD0A13E114F027AB9AC3F32CA696A0'

        if self.skip:
            raise unittest.SkipTest("Couldn't import Stem.")

    def writeTestDescriptorsToFile(self, filename, *descriptors):
        """Write **descriptors** to **filename**.

        :param str filename: A filename. It will be appended to the current
            working directory automatically.
        :param str descriptors: Some optional strings containing
            descriptors. Each one will be written to **filename** as-is.
        :rtype: str
        :returns: The full path to the file which was written to.
        """
        descFilename = os.path.join(os.getcwd(), filename)
        with open(descFilename, 'w') as fh:
            for desc in descriptors:
                fh.write(desc)
                fh.flush()
        return descFilename

    def test_parse_descriptors_parseServerDescriptorsFile(self):
        """Test for ``b.p.descriptors.parseServerDescriptorsFile``."""
        descFile = io.BytesIO(BRIDGE_SERVER_DESCRIPTOR)
        routers = descriptors.parseServerDescriptorsFile(descFile)
        self.assertIsInstance(routers, list)
        bridge = routers[0]
        self.assertIsInstance(bridge, RelayDescriptor)
        self.assertEqual(bridge.address, self.expectedIPBridge0)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_return_type(self):
        """``b.p.descriptors.parseNetworkStatusFile`` should return a dict."""
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0)
        routers = descriptors.parseNetworkStatusFile(descFile)
        self.assertIsInstance(routers, list)

    def test_parse_descriptors_parseNetworkStatusFile_has_RouterStatusEntryV2(self):
        """The items in the dict returned from
        ``b.p.descriptors.parseNetworkStatusFile`` should be
        ``RouterStatusEntryV2``s.
        """
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0)
        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]
        self.assertIsInstance(bridge, RouterStatusEntryV3)

    def test_parse_descriptors_parseNetworkStatusFile_one_file(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with one bridge
        networkstatus descriptor.
        """
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0)
        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]
        self.assertEqual(bridge.address, self.expectedIPBridge0)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_two_files(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with two bridge
        networkstatus descriptors.
        """
        expectedIPs = [self.expectedIPBridge0, self.expectedIPBridge1]

        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   BRIDGE_NETWORKSTATUS_0,
                                                   BRIDGE_NETWORKSTATUS_1)
        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]

        self.assertIn(bridge.address, expectedIPs)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_bad_nickname(self):
        """``b.p.descriptors.parseNetworkStatusFile`` with a bridge
        networkstatus descriptor which has a nickname that is too long should
        raise InvalidRouterNickname.
        """
        unparseable = BRIDGE_NETWORKSTATUS_0.replace(
            'MiserLandfalls',
            'MiserLandfallsWaterfallsSnowfallsAvalanche')
        # Write the descriptor to a file for testing. This is necessary
        # because the function opens the networkstatus file to read it.
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   unparseable)
        self.assertRaises(descriptors.InvalidRouterNickname,
                          descriptors.parseNetworkStatusFile,
                          descFile)

    def test_parse_descriptors_parseNetworkStatusFile_IPv6_ORAddress(self):
        """A Bridge can't have its primary ORAddress be IPv6 without raising
        a ValueError.
        """
        unparseable = BRIDGE_NETWORKSTATUS_0.replace(
            '2.215.61.223', '[2837:fcd2:387b:e376:34c:1ec7:11ff:1686]')
        descFile = self.writeTestDescriptorsToFile('networkstatus-bridges',
                                                   unparseable)
        self.assertRaises(ValueError,
                          descriptors.parseNetworkStatusFile,
                          descFile)

    def test_parse_descriptors_parseNetworkStatusFile_with_annotations(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with some document
        headers before the first 'r'-line.
        """
        expectedIPs = [self.expectedIPBridge0, self.expectedIPBridge1]
        descFile = 'networkstatus-bridges'

        with open(descFile, 'w') as fh:
            fh.write('signature and stuff from the BridgeAuth would go here\n')
            fh.write('some more annotations with parameters and stuff\n')
            fh.write(BRIDGE_NETWORKSTATUS_0)
            fh.write(BRIDGE_NETWORKSTATUS_1)
            fh.flush()

        routers = descriptors.parseNetworkStatusFile(descFile)
        bridge = routers[0]
        self.assertIn(bridge.address, expectedIPs)
        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_parseNetworkStatusFile_with_annotations_no_skipping(self):
        """Test ``b.p.descriptors.parseNetworkStatusFile`` with some
        document headers before the first 'r'-line, but without skipping said
        annotations.
        """
        expectedIPs = [self.expectedIPBridge0, self.expectedIPBridge1]
        descFile = 'networkstatus-bridges'

        with open(descFile, 'w') as fh:
            fh.write('signature and stuff from the BridgeAuth would go here\n')
            fh.write('some more annotations with parameters and stuff\n')
            fh.write(BRIDGE_NETWORKSTATUS_0)
            fh.write(BRIDGE_NETWORKSTATUS_1)
            fh.flush()

        self.assertRaises(ValueError,
                          descriptors.parseNetworkStatusFile,
                          descFile, skipAnnotations=False)

    def test_parse_descriptors_parseExtraInfoFiles_return_type(self):
        """The return type of ``b.p.descriptors.parseExtraInfoFiles``
        should be a dictionary (after deduplication).
        """
        descFile = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFile)
        self.assertIsInstance(routers, dict)

    def test_parse_descriptors_parseExtraInfoFiles_has_BridgeExtraInfoDescriptor(self):
        """The return of ``b.p.descriptors.parseExtraInfoFiles`` should
        contain ``BridgeExtraInfoDescriptor``s.
        """
        descFile = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFile)
        bridge = routers.values()[0]
        self.assertIsInstance(bridge, RelayExtraInfoDescriptor)

    def test_parse_descriptors_parseExtraInfoFiles_one_file(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with only one
        bridge extrainfo file.
        """
        descFile = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFile)
        bridge = routers.values()[0]

        # The number of transports we parsed should be equal to the number of
        # 'transport' lines in the descriptor:
        self.assertEqual(len(bridge.transport),
                         BRIDGE_EXTRA_INFO_DESCRIPTOR.count('transport '))

        self.assertEqual(bridge.fingerprint, self.expectedFprBridge0)

    def test_parse_descriptors_deduplicate_identical_timestamps(self):
        """Parsing two descriptors for the same bridge with identical
        timestamps should log a ``b.p.descriptors.DescriptorWarning``
        and retain only one copy of the descriptor.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFileOne, descFileTwo)

        self.assertEqual(len(routers), 1)

    def test_parse_descriptors_parseExtraInfoFiles_two_files(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with two
        bridge extrainfo files, and check that only the newest extrainfo
        descriptor is used.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE)
        routers = descriptors.parseExtraInfoFiles(descFileOne, descFileTwo)

        # We shouldn't have duplicates:
        self.assertEqual(len(routers), 1,
                         "We shouldn't have any duplicate descriptors.")

        # We should only have the newest descriptor:
        bridge = routers.values()[0]
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-11-04 08:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def test_parse_descriptors_parseExtraInfoFiles_two_files_reverse(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with two bridge
        extrainfo files. This time, they are processed in reverse to ensure
        that we only keep the newer duplicates of descriptors, no matter what
        order they appeared in the files.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFileOne, descFileTwo)

        self.assertEqual(len(routers), 1,
                         "We shouldn't have any duplicate descriptors.")

        bridge = routers.values()[0]
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-11-04 08:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def test_parse_descriptors_parseExtraInfoFiles_three_files(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with three
        bridge extrainfo files, and check that only the newest extrainfo
        descriptor is used.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWER_DUPLICATE)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileThree = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  descFileTwo,
                                                  descFileThree)

        # We shouldn't have duplicates:
        self.assertEqual(len(routers), 1,
                         "We shouldn't have any duplicate descriptors.")

        # We should only have the newest descriptor:
        bridge = routers.values()[0]
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-12-04 03:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def test_parse_descriptors_parseExtraInfoFiles_no_validate(self):
        """Test for ``b.p.descriptors.parseExtraInfoFiles`` with
        descriptor validation disabled.
        """
        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  validate=False)
        self.assertGreaterEqual(len(routers), 1)

    def test_parse_descriptors_parseExtraInfoFiles_unparseable(self):
        """Test parsing three extrainfo descriptors: one is a valid descriptor,
        one is an older duplicate, and one is unparseable (it has a bad
        geoip-db-digest line). There should be only one descriptor returned
        after parsing.
        """
        # Give it a bad geoip-db-digest:
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "DontParseMe F373CC1D86D82267F1F1F5D39470F0E0A022122E").replace(
                "geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F",
                "geoip-db-digest FOOOOOOOOOOOOOOOOOOBAAAAAAAAAAAAAAAAAARR")

        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileThree = self.writeTestDescriptorsToFile(
            "unparseable-descriptor", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  descFileTwo,
                                                  descFileThree)
        self.assertIsInstance(routers, dict)
        self.assertEqual(len(routers), 1, (
            "There were three extrainfo descriptors: one was a duplicate, "
            "and one was unparseable, so that should only leave one "
            "descriptor remaining."))

        bridge = routers.values()[0]
        self.assertEqual(
            bridge.fingerprint,
            "E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            ("It looks like the (supposedly) unparseable bridge was returned "
             "instead of the valid one!"))
        self.assertEqual(
            bridge.published,
            datetime.datetime.strptime("2014-12-04 03:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

    def test_parse_descriptors_parseExtraInfoFiles_unparseable_and_parseable(self):
        """Test parsing four extrainfo descriptors: two are valid descriptors,
        one is an older duplicate of one of the valid descriptors, and one is
        unparseable (it has a line we shouldn't recognise). There should be
        only two descriptors returned after parsing.
        """
        # Mess up the bridge-ip-transports line:
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "DontParseMe F373CC1D86D82267F1F1F5D39470F0E0A022122E").replace(
                "bridge-ip-transports <OR>=8",
                "bridge-ip-transports <OR>")

        parseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "ImOkWithBeingParsed 2B5DA67FBA13A6449DE625673B7AE9E3AA7DF75F")

        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileThree = self.writeTestDescriptorsToFile(
            "unparseable-descriptor.new", unparseable)
        descFileFour = io.BytesIO(parseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne,
                                                  descFileTwo,
                                                  descFileThree,
                                                  descFileFour)
        self.assertIsInstance(routers, dict)
        self.assertEqual(len(routers), 2, (
            "There were four extrainfo descriptors: one was a duplicate, "
            "and one was unparseable, so that should only leave two "
            "descriptors remaining."))

        self.assertNotIn("F373CC1D86D82267F1F1F5D39470F0E0A022122E", routers.keys(),
                         "The 'unparseable' descriptor was returned by the parser.")

        self.assertIn("E08B324D20AD0A13E114F027AB9AC3F32CA696A0", routers.keys(),
            ("A bridge extrainfo which had duplicates was completely missing "
             "from the data which the parser returned."))
        self.assertEqual(
            routers["E08B324D20AD0A13E114F027AB9AC3F32CA696A0"].published,
            datetime.datetime.strptime("2014-12-04 03:10:25", "%Y-%m-%d %H:%M:%S"),
            "We should have the newest available descriptor for this router.")

        self.assertIn("2B5DA67FBA13A6449DE625673B7AE9E3AA7DF75F", routers.keys(),
                      "The 'parseable' descriptor wasn't returned by the parser.")

    def test_parse_descriptors_parseExtraInfoFiles_bad_signature_footer(self):
        """Calling parseExtraInfoFiles() with a descriptor which has a
        signature with a bad "-----END SIGNATURE-----" footer should return
        zero parsed descriptors.
        """
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            '-----END SIGNATURE-----',
            '-----END SIGNATURE FOR REALZ-----')
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileOne = self.writeTestDescriptorsToFile(
            "bad-signature-footer", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne)

        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_parseExtraInfoFiles_missing_signature(self):
        """Calling parseExtraInfoFiles() with a descriptor which is
        missing the signature should return zero parsed descriptors.
        """
        # Remove the signature
        BEGIN_SIG = '-----BEGIN SIGNATURE-----'
        unparseable, _ = BRIDGE_EXTRA_INFO_DESCRIPTOR.split(BEGIN_SIG)
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileOne = self.writeTestDescriptorsToFile(
            "missing-signature", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne)

        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_parseExtraInfoFiles_bad_signature_too_short(self):
        """Calling _verifyExtraInfoSignature() with a descriptor which has a
        bad signature should raise an InvalidExtraInfoSignature exception.
        """
        # Truncate the signature to 50 bytes
        BEGIN_SIG = '-----BEGIN SIGNATURE-----'
        doc, sig = BRIDGE_EXTRA_INFO_DESCRIPTOR.split(BEGIN_SIG)
        unparseable = BEGIN_SIG.join([doc, sig[:50]])
        # This must be a "real" file or _copyUnparseableDescriptorFile() will
        # raise an AttributeError saying:
        # '_io.BytesIO' object has no attribute 'rpartition'"
        descFileOne = self.writeTestDescriptorsToFile(
            "truncated-signature", unparseable)
        routers = descriptors.parseExtraInfoFiles(descFileOne)

        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_parseExtraInfoFiles_unparseable_BytesIO(self):
        """Test parsing three extrainfo descriptors: one is a valid descriptor,
        one is an older duplicate, and one is unparseable (it has a bad
        geoip-db-digest line). The parsing should raise an unhandled
        AttributeError because _copyUnparseableDescriptorFile() tries to
        manipulate the io.BytesIO object's filename, and it doesn't have one.
        """
        # Give it a bad geoip-db-digest:
        unparseable = BRIDGE_EXTRA_INFO_DESCRIPTOR.replace(
            "MiserLandfalls E08B324D20AD0A13E114F027AB9AC3F32CA696A0",
            "DontParseMe F373CC1D86D82267F1F1F5D39470F0E0A022122E").replace(
                "geoip-db-digest 09A0E093100B279AD9CFF47A67B13A21C6E1483F",
                "geoip-db-digest FOOOOOOOOOOOOOOOOOOBAAAAAAAAAAAAAAAAAARR")

        descFileOne = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR)
        descFileTwo = io.BytesIO(BRIDGE_EXTRA_INFO_DESCRIPTOR_NEWEST_DUPLICATE)
        descFileThree = io.BytesIO(unparseable)
        self.assertRaises(AttributeError,
                          descriptors.parseExtraInfoFiles,
                          descFileOne, descFileTwo, descFileThree)

    def test_parse_descriptors_parseExtraInfoFiles_empty_file(self):
        """Test parsing an empty extrainfo descriptors file."""
        routers = descriptors.parseExtraInfoFiles(io.BytesIO(''))
        self.assertIsInstance(routers, dict)
        self.assertEqual(len(routers), 0)

    def test_parse_descriptors_copyUnparseableDescriptorFile_return_value(self):
        """``b.p.descriptors._copyUnparseableDescriptorFile()`` should return
        True when the new file is successfully created.
        """
        filename = "bridge-descriptors"
        with open(filename, 'w') as fh:
            fh.write(BRIDGE_SERVER_DESCRIPTOR)
            fh.flush()

        result = descriptors._copyUnparseableDescriptorFile(filename)
        self.assertTrue(result)  # should return True

    def test_parse_descriptors_copyUnparseableDescriptorFile_new_filename(self):
        """``b.p.descriptors._copyUnparseableDescriptorFile()`` should create a
        copy of the bad file with a specific filename format.
        """
        filename = "bridge-descriptors"
        with open(filename, 'w') as fh:
            fh.write(BRIDGE_SERVER_DESCRIPTOR)
            fh.flush()

        descriptors._copyUnparseableDescriptorFile(filename)
        matchingFiles = glob.glob("*_bridge-descriptors.unparseable")
        self.assertEqual(len(matchingFiles), 1)

        newFile = matchingFiles[-1]
        self.assertTrue(os.path.isfile(newFile))

        timestamp = datetime.datetime.strptime(newFile.split("_")[0],
                                               "%Y-%m-%d-%H:%M:%S")
        # The timestamp should be roughly today (unless we just passed
        # midnight, then it might be +/- 1):
        self.assertApproximates(timestamp.now().day, timestamp.day, 1)

        # The timestamp should be roughly this hour (+/- 1):
        self.assertApproximates(timestamp.now().hour, timestamp.hour, 1)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, Isis Lovecruft
# :license: see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.geo` module."""

import ipaddr

from twisted.trial import unittest

from bridgedb import geo


class GeoTests(unittest.TestCase):
    """Unittests for the :mod:`bridgedb.geo` module."""

    def setUp(self):
        self._orig_geoip = geo.geoip
        self._orig_geoipv6 = geo.geoipv6

        self.ipv4 = ipaddr.IPAddress('38.229.72.16')
        self.ipv6 = ipaddr.IPAddress('2620:0:6b0:b:1a1a:0:26e5:4810')

        # WARNING: this is going to fail if the torproject.org A and AAAA
        # records above are reassigned to another host, or are randomly
        # geolocated to somewhere other than where they are currently
        # geolocated (in the US):
        self.expectedCC = 'US'

    def tearDown(self):
        geo.geoip = self._orig_geoip
        geo.geoipv6 = self._orig_geoipv6

    def test_geo_getCountryCode_ipv4_str(self):
        """Should return None since the IP isn't an ``ipaddr.IPAddress``."""
        self.assertIsNone(geo.getCountryCode(str(self.ipv4)))

    def test_geo_getCountryCode_ipv4_no_geoip_loopback(self):
        """Should return None since this IP isn't geolocatable (hopefully ever)."""
        ipv4 = ipaddr.IPAddress('127.0.0.1')
        self.assertIsNone(geo.getCountryCode(ipv4))

    def test_geo_getCountryCode_ipv4_class(self):
        """Should return the CC since the IP is an ``ipaddr.IPAddress``."""
        cc = geo.getCountryCode(self.ipv4)
        self.assertIsNotNone(cc)
        self.assertIsInstance(cc, basestring)
        self.assertEqual(len(cc), 2)
        self.assertEqual(cc, self.expectedCC)

    def test_geo_getCountryCode_ipv6_str(self):
        """Should return None since the IP isn't an ``ipaddr.IPAddress``."""
        self.assertIsNone(geo.getCountryCode(str(self.ipv6)))

    def test_geo_getCountryCode_ipv6_no_geoip_record(self):
        """Should return None since this IP isn't geolocatable (yet)."""
        ipv6 = ipaddr.IPAddress('20::72a:e224:44d8:a606:4115')
        self.assertIsNone(geo.getCountryCode(ipv6))

    def test_geo_getCountryCode_ipv6_no_geoip_link_local(self):
        """Should return None since this IP isn't geolocatable (hopefully ever)."""
        ipv6 = ipaddr.IPAddress('ff02::')
        self.assertIsNone(geo.getCountryCode(ipv6))

    def test_geo_getCountryCode_ipv6_no_geoip_loopback(self):
        """Should return None since this IP isn't geolocatable (hopefully ever)."""
        ipv6 = ipaddr.IPAddress('::1')
        self.assertIsNone(geo.getCountryCode(ipv6))

    def test_geo_getCountryCode_ipv6_class(self):
        """Should return the CC since the IP is an ``ipaddr.IPAddress``."""
        cc = geo.getCountryCode(self.ipv6)
        self.assertIsNotNone(cc)
        self.assertIsInstance(cc, basestring)
        self.assertEqual(len(cc), 2)
        self.assertEqual(cc, self.expectedCC)

    def test_geo_getCountryCode_no_geoip(self):
        """When missing the geo.geoip database, getCountryCode() should return
        None.
        """
        geo.geoip = None
        self.assertIsNone(geo.getCountryCode(self.ipv4))

    def test_geo_getCountryCode_no_geoipv6(self):
        """When missing the geo.geoipv6 database, getCountryCode() should
        return None.
        """
        geo.geoipv6 = None
        self.assertIsNone(geo.getCountryCode(self.ipv4))



#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-clause BSD, see included LICENSE for information

from ._version import get_versions
from ._langs import get_langs

__version__ = get_versions()['version']
__langs__ = get_langs()

del get_versions
del get_langs



# -*- coding: utf-8 -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2014-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________


"""All available ``zope.interface``s in BridgeDB."""


from bridgedb.bridgerequest import IRequestBridges
from bridgedb.captcha import ICaptcha
from bridgedb.schedule import ISchedule
